/*
 *  A button with a drop down menu.
 *
 *  Unlike the previous version, this version uses the menuSelect widget.
 */
define(['jquery', 'bus', 'jquery_ui/selectmenu', 'perfect-scrollbar'], function($, Bus) {

	$.widget('ui.dateParameterMenu', $.ui.selectmenu, {
		_renderItem: function( ul, item ) {
			var label = item.label.split(' ');

			return ul
				.li(item.disabled && '.ui-state-disabled')
				.html('<span class="day">' + label.shift() + ' ' + label.shift() + '</span> ' + label.join(' '));
		}
	});

	$.widget('ui.chartParameterMenu', {
		options: {
			name: 'missing',
			title: 'Untitled',
			value: null,
			values: [],
			labels: null,
			bus: Bus(),
			select: null,           // function to mark the correct menu item dom element as selected. Is a temporary workaround for base_time parameter being in unusual format.
			on_click: $.noop
		},

		_create: function(){
			var self = this, o = self.options;
			self.selected_value = o.value || null;

			if(o.value && !~$.inArray(o.value, o.values)) console.warn('chartParameterMenu: invalid value', o.value);

			var widget = o.name === 'base_time' ? 'dateParameterMenu' : 'selectmenu';

			o.labels || (o.labels = o.values);

			var menu = self.element.addClass('parameter-selector')
				.formSelect(self._make_items())
				// TODO setting value here is having no effect
				.val(o.value)
				/*
				.each(function(){
					console.log('* chartParameterMenu: this=', $(this).find('option'));
					$(this).find('option[value="' + o.value + '"]').attr('selected', 'selected');
				})
				*/
				[widget]({
					select: function(event, ui) {
						console.log('select.select: ui.item=', ui.item);
						o.on_click(event, ui, o.name);
						self.select(ui.item.element);
					},
					open: function(ev) {
						menu.menu
							.parent()
							.css({maxHeight: Math.max(40, $(window).height() - menu.menu.offset().top - 10)})
							.perfectScrollbar({suppressScrollX: true, wheelSpeed: ECMWF.wheelspeed || 10})
							.perfectScrollbar('update');
					}
				})
				.each(function() {
					$(this).parent().find('label').prependTo(self.element.parent());
				})
				[widget]('instance');

			menu.menu.on('click', 'li', function(event) {
				var li = $(this);
				var i = li.index();
				var item = {index: i, value: self.element.find('select option:nth-child(' + (i + 1) + ')').attr('value'), label: li.text(), element: li};
				menu._select(item, event);
			});

			// TODO this version does use data-value
			o.bus.listen('chart-content-update', function(_, msg) {
				var instance = msg.product;
				if(self.selected_value !== instance.variables[o.name].value){
					self.selected_value = instance.variables[o.name].value;
					self.select($(this).find("[data-value='" + self.selected_value + "']"));
				}
			});
		},

		_destroy: function(){
			this.element.removeClass('parameter-selector');
		},

		setItems: function(parameter) {
			var o = this.options;

			o.values = parameter.values;
			o.labels = parameter.labels || [];
			this.menu_items = null;
			this.menu && this.menu.remove();
		},

		select_by_value: function() {
			// TODO may need to be updated for selectMenu
			var self = this, o = this.options;
			console.warn('chartParameterMenu.select_by_value: NEVER GET HERE');

			if(o.select){
				o.select();
				return;
			}
			self.selected_value && self.select(this.element.find("[data-value='" + self.selected_value + "']").orelse(function(){ console.log('chartParameterMenu: value not found: ', self.selected_value); }));
		},

		select: function(el) {
			el.parent().find('span.ui-icon').remove();
			el.span('.ui-icon ui-icon-check');
		},

		_make_items: function() {
			var self = this, o = self.options;

			return this.menu_items = {
				options: $.map(o.values, function(a, i) {
					return [[a, o.labels[i] || a]];
				}),
				label: o.title
			}
		}
	});
});
