/*
 *  Chart Application
 */
define(['config', 'jquery', 'bus', 'user_info', 'utils', (true ? 'grid/products' : 'www_chart/model/basic_instance_list'), 'product_browser/chart_handler', './router', 'filterable-list', 'grid/filters/path'], function(ECMWF, $, Bus, User, Utils, products, chart_handlers, router, list, path_filter) {

	var CH = chart_handlers;
	var login_warning_done = false;

	return {
		config: {
			page: {
				borderx: 52 // 4em from css
			},
			col1: {
				width: [[10000, 264], [1280, 220], [1024, 200], [700, 0]] // except for the last, these must match stylesheet
			},
			panel: {
				collapsible: false
			}
		},

		col1: function(o) {
			var self = this;

			// remove admin menu causing layout problems
			function remove_admin() {
				$('body').removeClass('admin-menu');
				$('#admin-menu').remove();
			}
			remove_admin();
			setTimeout(remove_admin, 1000);

			/*
			$('.top-bar, .footer-bar').appendTo(
				$('body').div('.scroll-wrapper')
					.perfectScrollbar({wheelSpeed: ECMWF.wheelspeed})
					.on('mouseenter', '.ps-container', function() {
						$(this).parents('.ps-container').find('.ps-scrollbar-y:first').css({opacity: 0.1});
					})
					.on('mouseleave', '.ps-container', function() {
						$(this).parents('.ps-container').find('.ps-scrollbar-y:first').css({opacity: ''});
					})
			);
			*/

			this.have_url = $.Deferred();

			this._package && products.set_package(this._package);

			list.subscribe(function() {
				if(!products.products.length && !login_warning_done){
					User.info.then(function(user) {
						user.uid || require(['dialogue2013'], function(Dialogue) {
							var url = '/auth/login?back=' + location.href;

							new Dialogue({
								id: 'no-products-warning',
								title: '<h2>You are not logged in</h2>',
								html: '<p>Please note that many charts have restricted access.</p><p>If you are an ECMWF partner, please <a href="'+ url +'">log in</a> to get access to a larger range of products.</p><p>If the content you are trying to view is for licensed users only, such as a specific chart, you might still not be granted access once logged in'
							});
							$('#no-products-warning').modal('show');
						});
						login_warning_done = true;
					})
				}
			});

			self.have_chart = $.Deferred(function(df) {
				Bus().listen('chart-added', function() {
					df.resolve();
				});
			})
		},

		scroll_to_col3: function() {
		},

		change_package: function(pkg) {
			var self = this;
			products.set_package(pkg);
			list.set_collection(products, function() {
				self._package = pkg;
			});
		},

		on_url_change: function(ctx, c) {
			var self = this, new_product, handler = chart_handlers.basic, prev = {};

			// note that we do not use chart_handler.state here because it cannot be guaranteed not to have already been updated unless the chart handler is called AFTER and VIA the url change.
			// this does NOT preserve the product on landing pages as originally intended.
			prev.name = router._path && router._path.page;
			prev.product = prev.name && products.product_by_name(prev.name);

			var is_landing_page = !ctx.params.product;

			// note difference betweeen product and page_product: product should be the currently selected product, even when on a landing page.
			var changed = $.extend({
				product: {
					widget: ctx.params && ((ctx.params.product && CH.state.list_product && ctx.params.product !== prev.name) || (CH.state.list_product && !ctx.params.product))
				},
				page_product: ctx.params.product !== (self.prev && self.prev.product),
				path: path_changed(prev.path, ctx.params.product_path)
			}, c);

			if(changed.pkg){
				console.log('>>> on_url_change: package has changed!', this._package, ' --> ', [ctx.params['package']]);

				self.change_package(ctx.params['package']);
			}
else {

			if(changed.product.url){
				console.log('%con_url_change: product has changed!', 'background-color:#bdf', [prev.name], ' --> ', [ctx.params.product]);
				if(ctx.params.product){
					list.have_items.then(function() {
						if((new_product = products.product_by_name(ctx.params.product))){
							CH.state.params = ctx.query;
							self.on_product_select(new_product, ctx.query);

							list.selection.set(list.lookup_idx_by_name(ctx.params.product));
						}else{
							if(ctx.canonicalPath.replace(/\?.*/, '') !== router.original_path){
								console.log('assuming external url. exiting application...');
								location.href = ctx.canonicalPath;
								return;
							}
							console.log('product not found but path is unchanged. not exiting');

							require(['dialogue2013'], function(Dialogue) {
								new Dialogue({
									id: 'product-not-found',
									title: '<h2>Product not found</h2>',
									html: '<p>Could not find the product <i>' + ctx.params.product + '</i>.</p>' + (!!~$.inArray(ctx.params.product, ['obstat', 'swdfp', 'medium']) ? '<p><a href="' + ctx.pathname + '/">' + ECMWF.ucfirst(ctx.params.product) + ' package</a></p>' : '')
								});
								$('#product-not-found').modal('show').click(function() {
									$('#product-not-found').modal('hide');
								});
							});
						}
					});
				}else{
					self.scroll_to_col2();
				}
			}

			if(ctx.params.product && changed.product.url){
				self.scroll_to_col3();
			}

			if(changed.path){
				path_filter.set([router.params.product_path]);
				self.update_category_links();
			}

			if(changed.path || changed.page_product || (!ctx.params.product && !self.is_landing_page) || (ctx.params.product && self.is_landing_page)){

				if(false){
					var url = [ctx.params.product_path.join('/')];
				}else{
					url = []; // package not in the content url
				}
				ctx.params.product && url.push(ctx.params.product);

				// page content
				this.config.enable_text && products.content(url.join('/'), function(result) {
					if(result && result.id){
						// cmf repo
						self.widgets.content.html('<h2>' + result.title + '</h2><div>' + result.body + '</div>');
						self.widgets.content.find('img').each(function() {
							var img = $(this);
							var src = img.attr('src');
							src.indexOf('ttp') === 1 || img.attr('src', ECMWF.options.ajax.chart_content.replace('/en', '') + src); // make url absolute
						});
					}else if(result && result.result){
						// drupal repo
						console.log('content: result=', result.result);
						self.widgets.content.html(result.result);
					}else{
						console.log('on_url_change: content: using fallback: changed.page_product=', changed.page_product);
						if(is_landing_page){
							self.widgets.content.empty();
						}else{
							// temporary page content from product database
							//if(changed.page_product){ // TODO PUTMEBACK changed.page_product is sometimes not set when it should be
								products.content_temp(router.params.product, function(result) {
									self.have_chart.done(function() {
										self.widgets.content.html(result.text || result.description)
									});
								});
							//}
						}
					}
				});
			}
}

			self.is_landing_page = is_landing_page;

			if(handler.instance){
				handler.instance.set_parameters(ctx.query);

				handler.instance.have.availability.done(function() {
					$.extend(CH.state.params, Utils.map(handler.instance.variables, function(v){ return v.to_string(); }));
				});
			}

			self.have_url.resolve();

			// TODO can we not use path_filter.subscribe ?
			function path_changed(old_path, new_path) {
				if(old_path && !new_path) return true;
				if(!old_path && new_path) return true;
				if(old_path.length !== new_path.length) return true;
				for(var i in new_path){
					if(old_path[i] !== new_path[i]) return true;
				}
				return false;
			}
		},

		on_product_select: function(product, params) {
			var self = this;

			if(!product.data || !product.data.package) return console.error('on_product_select: unexpected product format');

			// we don't get a loaded product here so cannot call product methods
			if(product.has_widget('epsgram')){
				list.remove_filter('facets');
			}
		},

		h1: function(text) {
			text &&
				$('h1')
					.text(text)
					.addClass('ready');
		},

		_on_resize: function(size) {
			// Increase the page width if the chart is wide.
			// Basic charts only.
			// This is a hack. There is no way this can work reliably in all situations.

			// TODO merge with do_layout ?

			// note that this needs to respond to resizing of both the window and the chart.

			var self = this;

			if(!CH.widgets.chart) return;

			size = size || {width: Math.floor(CH.widgets.chart.find('img').width() / 32) * 32};

			var window_width = $(window).width();
			var col1_width = this._col1_width();
			var left_width = col1_width + 186;
			if(true || (size.width > window_width - left_width - 10)){
				//if(!self._width_override){
					console.log('page.size-changed: overriding page width: ', size.width, window_width - left_width - 10);
					//self.size_override_targets.css({width: size.width + left_width});
					self._width_override = size.width;
					self.do_layout();
				//}
			}else{
				if(self._width_override){
					console.log('page.size-changed: removing size override...');
					self.size_override_targets.css({minWidth: ''});
					self._width_override = null;
					self.do_layout();
				}
			}
			$('#time-navigator').triggerHandler('container');
		},

		update_category_links: function() {
		},

		_col1_width: function() {
			var window_width = window.innerWidth; // media queries widths include the scrollbar

			var a = (ECMWF.options.col1 && ECMWF.options.col1.width) || this.config.col1.width
			var w = a[0]
			for(var i in a){
				if(a[i][0] < window_width) break;
				w = a[i]
			}
			return w[1];
		},

		_bind: function(fn) {
			var self = this;
			return function() {
				fn.apply(self, arguments);
			};
		}
	};
});
