/*
 *   There is more than one product collection manager available:
 *     - grid/products (this is the one currently in use)
 *     - basic_chart/model/basic_instance_list
 */

define(['config', 'jquery', 'ajax_queue', 'utils', 'bus', 'cache', './modernizr'], function(ECMWF, $, Ajaxqueue, Utils, Bus, Cache, Modernizr) {

var ga = null;

ECMWF.options.ajax.backend.print_chart || (ECMWF.options.ajax.backend.print_chart = '/plots/get-map/'); // temporary while www is different.

$.extend(Ajaxqueue.options,
	{
		cache: Cache
	},
	Utils.same_origin() || {
		host: location.protocol + '//' + ECMWF.options.backend_host
	},
	Utils.same_origin() || ECMWF.options.have_cors || {
		dataType: 'jsonp'
	}
);

ECMWF.browser.msie9 && require(['./msie9']);

/*
 *  The page can be used either as traditional page centered in the middle, or as a full screen application.
 */
var centered = false; // TODO needs to be taken from application.yaml

if(!Modernizr.history) require('html5-history-api');
/*
Modernizr.localstorage || r.push('storage');
ECMWF.browser.msie8 && r.push('msie_fixes');
*/
require(['grid/products', 'grid/filters/path', './router', 'mixins', 'page/query', './panel', 'grid', 'facets', 'tooltip', 'tooltip/tipsy', 'throttle-debounce', 'messages', 'hover_intent'], function(products, path_filter, router, mixins, qs){

	router.original_path = location.pathname; // for loop detection

	/*
	 *  Parse the URL to determine BASE, APP, PACKAGE and PRODUCT.
	 *  The path must now include the base_path
	 */
	function app_tokens_by_path(path) {
		var r = {
			tokens: path.replace(/\/$/, '').split('/')
		}
		r.tokens.shift();

		var i = -1;
		if(ECMWF.chart_app && ECMWF.chart_app.base_url){
			i = ECMWF.chart_app.base_url.split('/').length - 1;
		}else if(r.tokens.length > 1 && r.tokens[1] === 'tcyclone'){
			i = 1;
		}else{
			// parse url without config (deprecated)
			var match = $.map(r.tokens, function(v, i){
				if(!!~$.inArray(v, ['charts', 'charts-beta', 'browser'])){
					return i;
				}
			});
			i = match.length ? match[0] : -1;
		}

		if(i > -1){
			r.app = r.tokens[i];

			if(r.tokens.length > i + 1 && (r.tokens[i + 1] !== 'all')) r.chart_package = r.tokens[i + 1];

			r.base_path = '/' + r.tokens.slice(0, i).join('/');
			if(r.base_path == '/') r.base_path = '';
		}

		return r;
	}

	/*
	 * Ongoing improvements to url parsing
	 *  - the application can now be at the top level (eg /charts/medium/ is ok)
	 *  - TODO allow the app name to be absent (eg allow /medium/)
	 *
	 */
	tokens = app_tokens_by_path(location.pathname);

	if(!tokens.app) throw new Error('cannot parse url');

	router.set_base_path(tokens.base_path);
	router.app_path = '/' + tokens.app + '/';

	// TODO move to application
	if(tokens.chart_package === 'tcyclone'){
		tokens.app = tokens.chart_package;
	}

	ECMWF.load_app(tokens.app, function(_page) {

		__DEV__ && Bus().send('event-start', ['page start']);

		$.getUserPreference = $.noop;
		ECMWF.wheelspeed = ECMWF.browser.mozilla ? 10 : 1;

		var main = $(ECMWF.options.dom.main || '#main');

		var col1 = main.find('.column-1');
		var col2 = main.find('.column-2');
		var col3 = main.find('.chart-column').orelse(function(){ return main.div('.column chart-column'); })

		if(!centered){
			col1.show();
		}

		var Page = function(def) {
			$.extend(this, def);
			this.widgets = {
				col1: col1,
				col2: col2,
				col3: col3
			};
			this.centered = centered;
			this.query = {};
			this._package = tokens.chart_package;

			this.show = function() {
				this.widgets.col2.panel($.extend({}, this.config.panel, this.config.col2.panel));
				this.widgets.col3.panel(this.config.panel);

				this.col1.call(page, {el: col1});
				this.col2.call(page, {el: col2});
				this.col3.call(page, {el: col3});
			}

			this.get_layout_info = function() {
				// these values represent (mostly) the original, not dynamic, sizes.

				var margin = 10;

				return {
					mode: (parseInt(main.css('minWidth'), 10) === 1060 ? 'normal' : 'wide'),
					col1: {
						outer_width: col1.outerWidth() + margin
					},
					col2: {
						inner_width: col2.width(),
						outer_width: col2.outerWidth() + 2 * margin
					},
					margin: margin
				}
			}
			this.layout = this.get_layout_info();
		}

		var page = new Page(_page);
		page.show();

		Bus().listen('size-changed', function(){ page.do_layout(); });

		$('body').container({layout: 'single'}).bind('layout', function(_, size) {
			page.layout.mode = (parseInt(main.css('minWidth'), 10) === 1060 ? 'normal' : 'wide');
			page.do_layout();
		});

	var contextual_links = [
		['/en/forecasts/charts/all', 'Charts'],
		['/en/forecasts/charts/tcyclone', 'Tropical cyclones'],
		['/plots/browser', 'Chart browser'],
		['http://eccharts.ecmwf.int/forecaster/', 'Eccharts'],
		['http://eccharts.ecmwf.int/dashboard/', 'Dashboard']

		/* menu from www
		['/en/forecasts/charts/catalogue/', 'Browse all charts']
		['/en/forecasts/charts/catalogue?f%5B0%5D=im_field_chart_type%3A481&amp;f%5B1%5D=im_field_chart_type_2%3A607', 'Medium range'],
		['/en/forecasts/charts/catalogue?f%5B0%5D=im_field_chart_type_2%3A607&amp;f%5B1%5D=im_field_chart_type%3A482', 'Extended range'],
		['/en/forecasts/charts/catalogue?f%5B0%5D=im_field_chart_type_2%3A607&amp;f%5B1%5D=im_field_chart_type%3A483', 'Long range'],
		['/forecasts/see-our-forecasts/medium-range-forecast-charts/monitoring-observing-system', 'Monitoring'],
		['/en/forecasts/charts/severe-weather-forecasting-demonstration-project-swfdp', 'Severe Weather Forecast'],
		['/en/forecasts/charts/commercial-chart-catalogue', 'Commercial chart catalogue']
		*/
	];

	/*
	$('#page-wrapper > .navigation').ul().each(function() {
		var ul = $(this);
		$.each(contextual_links, function(i, v) {
			ul.li().a({href: v[0]}).text(v[1]);
		});
	});
	*/

	function update_analytics(){
		ga && ga('send', 'pageview', document.location.href);
	}

	router
			.route('*', function parse(ctx, next) {
				ctx.query = qs.parse(ctx.querystring);
				next();
			})
			.route('/:app', function(ctx, next) {
				// on www this is the charts landing page
				if($('body.not-front').length && ctx.pathname !== router.original_path) location.href = ctx.pathname;
			})
			.route('*', function(ctx, next){
				// note that handling '*' interferes with proper 404 handling.

				var _tokens = app_tokens_by_path(tokens.base_path + ctx.params[0]);
				if(_tokens.app){
					var pkg = _tokens.chart_package;
					_tokens = _tokens.tokens;
					_tokens.shift();

					var is_product = ctx.params[0][ctx.params[0].length - 1] !== '/';

					ctx.params.product = (is_product && _tokens.pop()) || null;
					ctx.params.package = pkg;

					var changed = {
						pkg: pkg !== page._package,
						// TODO should perhaps use _new_path here as the semantics are clearer
						product: {url: (router._path.page || null) !== ctx.params.product},
						q: (function() {
							var a = qs.parse((router._path.q || '').replace('?', ''))
							var b = ctx.query
							var seen = {}
							var changed = $.map(
								$.merge(Object.keys(a), Object.keys(b)),
								function(v){ if(a[v] !== b[v] && !seen[v]){ seen[v] = true; return v; } }
							);
							console.log('ROUTE: qparam changed:', changed);
							return changed;
						})()
					}

					ctx.params.product_path = _tokens;
					router.params = ctx.params;

					//console.log('ROUTE: changed: ', router.path !== ctx.path, changed.product.url);
					($.isEmptyObject(router._path) || changed.product.url || changed.pkg || changed.q) && page.on_url_change(ctx, changed);
					set_breadcrumbs(ctx);

					// TODO is prev still needed? Is it something to do with product vs page_product?
					// In PageController, _new_path is the new path, and _path is the old.
					page.prev = {product: ctx.params.product};

					return;
				}

				//special case of redirecting to login page correctly
				//TODO should be a separate route.
				if (ctx.path.indexOf('/auth/login') > -1) {
					location.href = ctx.path;
					return;
				}

				console.log('route not found');
				location.href = ctx.canonicalPath;
			})
			.start();

		function set_breadcrumbs(ctx) {
			if(!ECMWF.chart_app || !page.config.enable_hierarchy) return;

			var path = ctx.params.product_path.slice(0);
			path.unshift('charts');
			var a = '';
			$('.breadcrumbs').html($.map(path, function(v){
				a += v + '/';
				var path = a.replace('charts/', '').replace(/\/$/, '');
				return '<a href="/' + ECMWF.chart_app.base_url.replace('charts', '') + a + '" data-path="' + path + '">' + v + '</a>';
			}).join(' &gt; '));

			var last = $('.breadcrumbs a:last').text();
			$('.breadcrumbs a:last').remove();
			$('.breadcrumbs').append(last);

			$('.breadcrumbs a').click(function() {
				path_filter.set([$(this).attr('data-path').split('/')]);
				return false;
			});
		}

		require(['shortcuts'], function() {
			var fullscreen = false, bus = Bus();

			$(document).shortcuts({
				shortcuts: [
					{ key: 70,                  signal: 'fullscreen'}, // 'f'
					{ key: $.ui.keyCode.LEFT,   signal: 'prev-step' },
					{ key: $.ui.keyCode.RIGHT,  signal: 'next-step' },
					{ key: $.ui.keyCode.COMMA,  signal: 'animation-speed-down' },
					{ key: $.ui.keyCode.PERIOD, signal: 'animation-speed-up' },

					{ key: 51, callback: function() { // '3'
						require(['chart/plugins/timers'], function(plugin) {
							var chart = $.fn.chart && $('.chart').chart('instance') || ($.fn.basicChart && $('.basic').basicChart('instance'));
							if(chart && chart.add_plugin('Timers', plugin)){
								require(['notify'], function() {
									$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Calltree timers are now enabled.<br/>New requests will have timer information available.'});
								});
							}else{
								Bus().send('open-calltree');
							}
						});
					}},
					{ key: 61, callback: function() { // '='
						require(['chart/plugins/timers'], function(plugin) {
							var chart = $.fn.chart && $('.chart').chart('instance');
							if(chart && chart.add_plugin('Timers', plugin)){
								chart.options.messageBus.clear().listen_once('chart-content-update', function() {
									bus.send('show-timers');
								});
								chart._redraw();
							}else{
								Bus().send('show-timers');
							}
						});
					}},
					{ key: 83, callback: function(){ // 's'
					}}
			]});

			bus.listen('fullscreen', function() {
				if(fullscreen){
					$('header, #footer').show();
					$('.column-1, .main-column-wrapper').css({top: '', bottom: ''});
					fullscreen = false;
				}else{
					$('header, #footer').hide();
					$('.column-1, .main-column-wrapper').css({top: 0, bottom: 0});
					fullscreen = true;
				}

				page.do_layout();
			});
		});
	});
});

});
