define(['config', 'jquery', 'bus', 'ajax_queue', 'framework'], function(ECMWF, $, Bus, Ajaxqueue) {

	var data, schema;

	function load_data(ready) {
		ready = ready || $.noop;

		if (data) {
			ready(data);
			return;
		}

		Ajaxqueue({
			url: ECMWF.options.ajax.backend.get_user_data,
			success: function(result) {
				data = result;
				ready(data);
			},
			error: function(result) {
				// Ignore for now ....
				data = {};
				ready(data);
			}
		});
	}

	function store_data(ready) {

		ready = ready || $.noop;

		delete data.schema;

		Ajaxqueue({
			url: ECMWF.options.ajax.backend.set_user_data,
			json: data,
			name: 'set_user_data',
			success: function(result) {
				data = result;
				ready();
			}
		});
	}

	function load_schema(ready) {
		ready = ready || function() {};

		if (schema) {
			ready(schema);
			return;
		}

		Ajaxqueue({
			url: ECMWF.options.ajax.backend.preferences_schema,
			type: 'GET',
			success: function(result) {
				if ($.isEmptyObject(result)) {
					if (!ECMWF.token) {
						Bus().send('log-error', 'Cannot load preferences');
						result = {};
					}
				}
				schema = result;
				ready(schema);
			},
			error: function(result) {
				if (!ECMWF.token) {
					Bus().send('log-error', 'Cannot load preferences');
				}
				schema = {};
				ready(schema);
			}
		});
	}

	$.frameworkRegister(load_schema, 'Load preference schema');
	$.frameworkRegister(load_data, 'Load user data');

	Bus().listen('preferences-changed', function(_, editable_prefs) {
		var old_prefs = data.preferences;
		data.preferences = editable_prefs;
		for (var i in old_prefs) {
			if (typeof data.preferences[i] === 'undefined' && schema[i] && schema[i].editable === false) {
				data.preferences[i] = old_prefs[i];
			}
		}
		store_data();
	});

	function set_data(kind, name, value) {
		if (!data[kind]) data[kind] = {};
		var old = data[kind][name];
		data[kind][name] = value;
		store_data();
		return old;
	}

	var warning_done;

	function get_data(kind, name, value) {
		if (!data) data = {};
		if (!data[kind]) data[kind] = {};
		if (typeof data[kind][name] === 'undefined') {
			data[kind][name] = value;
			if (typeof data[kind][name] === 'undefined') {
				//set default value from schema
				if (typeof schema === 'undefined') {
					warning_done || (console.warn('schema=', schema), warning_done = true);
					return null;
				}
				var s = schema[name];
				return s ? s.value : null;
			}
		}
		return data[kind][name];
	}

	$.setUserPreference = function(name, value) {
		return set_data('preferences', name, value);
	};

	$.getUserPreference = function(name, defaults) {
		return get_data('preferences', name, defaults);
	};

	$.eachUserPreference = function(callback) {
		data.preferences = data.preferences || {};
		for (var i in schema) {
			var pref = schema[i];
			if (typeof pref === 'object') {
				pref.value = (typeof data.preferences[pref.name] !== 'undefined') ? data.preferences[pref.name] : pref.value;
				callback(pref);
			}
		}
	};

	$.setUserSetting = function(name, value, workspace_name) {
		// name           : top-level key within 'settings'. If not set, value is assumed to be the whole settings data.
		// value          : the data to set
		// workspace_name : normally would be undefined, and the current workspace is used.

		var s = {};
		if (name) {
			s[name] = value;
		} else {
			s = value;
		}

		workspace_name = workspace_name || (window.Workspaces ? Workspaces.current : null);

		if (workspace_name) {
			console.log('>> setUserSetting: settingWorkspace: ', workspace_name, name, value);
			s.workspace_name = workspace_name;
			data.workspaces || (data.workspaces = {});
			data.workspaces[s.workspace_name] || console.log('>> setUserSetting: new workspace');
			var workspace = data.workspaces[s.workspace_name] || (data.workspaces[s.workspace_name] = {
				name: s.workspace_name,
				settings: {}
			});
			workspace.settings || (workspace.settings = {});
			if (name) workspace.settings[name] = value;
			else workspace.settings = value;

			if (Workspaces._workspaces[workspace_name]) {
				Workspaces._workspaces[workspace_name].settings || (Workspaces._workspaces[workspace_name].settings = {});
				if (name) Workspaces._workspaces[workspace_name].settings[name] = value;
				else Workspaces._workspaces[workspace_name].settings = value;
			}

		} else {
			data.settings = data.settings || {};
			if (name) data.settings[name] = value;
			else data.settings = value;
		}

		Ajaxqueue({
			url: ECMWF.options.ajax.backend.set_user_setting,
			json: s
		});
	};

	$.getUserSetting = function(name, defaults) {
		if (!data.workspaces || !data.current_workspace) {
			return get_data('settings', name, defaults);
		} else {
			var workspace = data.workspaces[data.current_workspace];
			if (workspace) {
				workspace.settings || (workspace.settings = {});
				return workspace.settings[name] || defaults;
			}
			return defaults;
		}
	};

	$.settings = {
		//most objects should be workspace-unaware; these functions are only for workspace implementors.
		_workspaces: {
			get: function(name) {
				return get_data('workspaces', name);
			},
			add: function(workspace) {
				(data.workspaces || (data.workspaces = {}))[workspace.name] = workspace;
			},
			select: function(name) {
				data.current_workspace = name;
				Ajaxqueue({
					// FIXME: dead code
					url: ECMWF.options.ajax.backend.set_workspaces_current,
					json: {
						workspace_name: name
					}
				});
			}
		}
	};

	$.getCurrentWorkspaceName = function() {
		return data.current_workspace;
	};

	$.getUserName = function(defaults) {
		return data ? data.owner : '';
	};
});
