/*
 *  in-application tests for pages using the dashboard widget
 *  (currently used for www.ecmwf.int/en/forecasts/dashboard)
 *
 *   TODO
 *     - fix 'Change Layout' test
 */
define(['jquery', 'bus', 'test-lib', './dashboard_set_testable', './dashboard_testable', 'application', 'user_info'], function($, Bus, Tests, DashboardSetTestable, DashboardTestable, Application, User){

	var dont_run = [
		/*
		'Cleanup',
		'Add Dashboard',
		'Add Widget',
		'Preview',
		'Edit widget',
		'Add Remove Widget',
		'Add Remove Many Widgets',
		'Remove Dashboard',
		'Move Widget',
		'Help panel',
		'Widget buttons',
		'Test meteograms',
		*/
		'Change Layout' // www currently doesn't have this feature
	]

	var bus = Bus();
		var dst = new DashboardSetTestable();
		Application.dashboard_ready.then(function(){
			var tests = new Tests([
				{
					name: 'Cleanup',
					start: function(){
						var test = dst.test = this;
						this.bus = bus;

						(function next(){
							var tabs = $('#dashboard-set .ui-tabs-nav a')
							if(tabs.length > 1){
								var untitled = $('#dashboard-set .ui-tabs-nav a:contains("Untitled"):first')
								if(untitled.length){
									var name = $(untitled[0]).attr('href').replace('#', '')
									dst.remove_dashboard(name).then(next);
								}else{
									test.finished()
								}
							}else{
								test.finished()
							}
						})();
					}
				},
				{
					name: 'Add Dashboard',
					start: function(){
						var test = dst.test = this;
						this.bus = bus;
						$.when(dst.add_dashboard()).then(test.finished);
					}
				},
				{
					name: 'Add Widget',
					start: function(){
						dst.test = this;
						$.when(new DashboardTestable(this).add_widget()).then(dst.test.finished);
					}
				},
				{
					name: 'Preview',
					start: function(){
						var test = dst.test = this;
						test.set_timeout(20000);
						var dt = new DashboardTestable(this);

						var dashboard = $('.dashboard .ui-tabs-panel:visible');
						// TODO for this test, the widget must be an interactive chart.
						dt.ensure_widget(function(widget){
							$.when(dt.multiple_previews(widget.attr('id'))).then(function(){
								test.assert(!$('.chart-controls').length);
								$.when(dt.multiple_previews(widget.attr('id'))).then(function(){
									dst.change_dashboard($('.ui-tabs-nav > li:first a').attr('href').slice(1)).then(test.bind(test.finished));
								});
							});
						});
					}
				},
				{
					/*
					 *   This test is currently very simple but serves to check that the form widget is loaded.
					 */
					name: 'Edit widget',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);

						dt.ensure_editable_widget(function(widget){
							$.when(dt.edit_start(widget.attr('id')))
								.pipe(dt.edit_stop(widget.attr('id')))
								.then(test.finished);
						});
					}
				},
				{
					name: 'Add Remove Widget',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);
						$.when(dt.add_widget())
							.pipe(function(widget){
								console.log('>>> on_added: widget=', widget);
								return dt.remove_widget(widget.options.name);
							})
							.then(test.finished);
					}
				},
				{
					name: 'Add Remove Many Widgets',
					start: function(){
						var n_widgets_to_add = 3;

						var test = dst.test = this;
						var dt = new DashboardTestable(this);

						var fnlist = [];
						for(var j = 0; j < n_widgets_to_add; j++) fnlist.push(dt.bind(dt.add_widget));

						// TODO rename all the widgets, not just the first
						fnlist.push(function(done){
							return $.Deferred(function(df){
								var widget = dt.first_widget();
								dt.rename_widget(widget, 'Add Remove ' + 1).then(function(){
									df.resolveWith(test);
									done();
								})
							}).promise();
						})

						var n_widgets = dt.n_widgets();

						console.log('_____________________________');
						test.series(fnlist, function(err){
							if(err) console.warn('err=', err); // TODO
							console.log('series A finished. n_widgets=', dt.n_widgets());

							var ok = dt.n_widgets() === n_widgets + n_widgets_to_add;
							test.assert(ok, 'n_widgets=' + dt.n_widgets() + ' expected ' + (n_widgets + n_widgets_to_add));
							if(!ok) return;

							var fnlist = [];
							for(var j = 0; j < n_widgets_to_add; j++) fnlist.push(dt.bind(dt.remove_widget));
							test.series(fnlist, function(err){
								console.log('series B finished. n_widgets=', dt.n_widgets());
								test.assert(dt.n_widgets() === n_widgets, 'n_widgets=' + dt.n_widgets() + ' expected ' + (n_widgets));
							});

							return false;
						});
					}
				},
				{
					name: 'Remove Dashboard',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);
						$.when(dst.remove_dashboard(dt.dashboard.name)).then(test.finished);
					}
				},
				{
					name: 'Move Widget',
					start: function(){
						var test = dst.test = this;
						this.bus = bus;
						var db1, db2, widget;
						$.when(Test.sleep)
							.then(dst.bind(dst.add_dashboard))
							.then(function(dashboard){ db1 = new DashboardTestable(test); return db1.add_widget(); })
							.then(function(){ return db1.rename('Dest') })
							.then(bind(dst.add_dashboard))
							.then(function(dashboard){ return (db2 = new DashboardTestable(test)).add_widget(); })
							.then(function(w){ widget = w; return db2.rename('Src') })
							.then(function(){ return db2.rename_widget(widget, 'To move 1') })
							.then(function(){
								console.log('%cmoving widget (1)', 'background-color:#9f9', db1.dashboard2, '-->', db2.dashboard2)
								return dst.move_widget(widget, db2, db1);
							})
							.then(function(){
								return $.Deferred(function(df){
									test.wait_for('.empty-dashboard-content', 'empty message').then(function(){ df.resolveWith(test) });
								}).promise();
							})
							.then(bind(dst.validate_active_dashboard))
							.then(bind(dst.remove_dashboard))
							.then(function(){
								return $.Deferred(function(df){
									var dest_widget = db1.dashboard2.widgets.columns[0].rows[0]
									test.assert(dest_widget.widget.name === widget.options.name, 'name')
									test.assert(dest_widget.widget.title === widget.options.title, 'title')
									test.assert(dest_widget.li.find('h3').text() === widget.options.title, 'h3')
									df.resolveWith(test);
								}).promise();
							})
							.then(bind(dst.validate_active_dashboard))

							// do now do a 2nd move
							.then(bind(dst.add_dashboard))
							.then(function(dashboard){ return (db2 = new DashboardTestable(test)).add_widget(); })
							.then(function(w){ widget = w; return db2.rename('Src2') })
							.then(function(){ return db2.rename_widget(widget, 'To move 2') })
							.then(function(){
								console.log('%cmoving widget (2)', 'background-color:#9f9', db1.dashboard2, '-->', db2.dashboard2)
								return dst.move_widget(widget, db2, db1);
							})
							.then(test.wait_for('.empty-dashboard-content', 'empty message'))
							.then(bind(dst.validate_active_dashboard))
							.then(bind(dst.remove_dashboard))

							.then(bind(dst.remove_dashboard))
							.then(test.finished);
					}
				},
				{
					name: 'Change Layout',
					start: function(){
						// TODO so that test is repeatable, we should create a new dashboard with a known number of widgets
						var test = dst.test = this;
						var dt = new DashboardTestable(this);
						$.when(dt.set_n_columns(4))
							.pipe(function(){ return dt.set_n_columns(6); })
							.pipe(function(){ return dt.set_n_columns(1); })
							.pipe(function(){ return dt.set_n_columns(3); })
							.then(test.finished);
					}
				},
				{
					/*
					 *   TODO fix check for interactive non-eccharts widget
					 */
					name: 'Widget buttons',
					start: function(){
						var test = dst.test = this;
						dst.add_dashboard().then(function(dashboard){
							var dt = new DashboardTestable(this);
							dt.rename('Widget buttons').then(function(){

								dt.add_widget(function(_, widget){
									dst.validate_active_dashboard().then(function(){
										dt.rename_widget(widget, 'Eccharts').then(function(){
											var el = widget.element;
											[
												['.ui-icon-help', 1],
												['.ui-icon-pencil', 1],
												['.ui-icon-extlink', (User.is_forecaster() ? 1 : 0)],
												['.ui-icon-triangle-1-n', 1],
												['.ui-icon-close', 1]
											].forEach(function(v, i){
												test.assert(el.find(v[0]).length === v[1], v[0] + ' got:' + el.find(v[0]).length + ' expected:' + v[1]);
											})

											dt.add_basic_widget(function(_, widget){
												dt.rename_widget(widget, 'Basic').then(function(){
													var el = widget.element;
													var icons = [
														['.ui-icon-help', 0],
														['.ui-icon-pencil', 0],
														['.ui-icon-extlink', 0],
														['.ui-icon-triangle-1-n', 1],
														['.ui-icon-close', 1]
													]
													icons.forEach(function(v, i){
														test.assert(el.find(v[0]).length === v[1], v[0] + ' got:' + el.find(v[0]).length + ' expected:' + v[1]);
													})

													dt.add_interactive_catalogue_widget(function(_, widget){
														var el = widget.element;
														var icons = [
															['.ui-icon-help', 1],
															['.ui-icon-pencil', 1],      // TODO is not supposed to be editable but there is no simple way to disable it for new widgets. However it doesnt matter because users cannot add this chart type from within the dashboard anyway.
															//['.ui-icon-extlink', 1],   // not able to test this currently. As of 2017.04, this icon is present when testing manually.
															['.ui-icon-triangle-1-n', 1],
															['.ui-icon-close', 1]
														]
														icons.forEach(function(v, i){
															test.assert(el.find(v[0]).length === v[1], v[0] + ' got:' + el.find(v[0]).length + ' expected:' + v[1]);
														})
														dst.remove_dashboard(dashboard.options.dashboard.name).then(function(){
															test.finished();
														})
													})
												})
											})
										})
									})
								})
							})
						})
					}
				},
				{
					name: 'Help panel',
					start: function(){
						var test = dst.test = this;
						dst.ensure_interactive_chart(function(){
							test.finished();
						})
					}
				},
				{
					name: 'Test meteograms',
					start: function(){
						var test = dst.test = this;
						var dt = new DashboardTestable(this);

						test.series([
							function(done){
								test.set_timeout(8000);
								dt.add_meteogram().then(done)
							},
							function(done){
								test.set_timeout(8000);
								dt.add_meteogram().then(done)
							},
							function(done){
								test.set_timeout(8000);
								dt.add_meteogram('meteogram_15_days').then(done)
							},
							function(done){
								test.set_timeout(8000);
								dt.dashboard2.widgets.all(function(w){
									dt.remove_widget(w.name).then(done)
									return false;
								})
							}
						])
					}
				}
			]);

			tests.dont_run(dont_run).run();
		});

		function bind(fn){
			return function(){ return fn.apply(dst, arguments); };
		}
});
