define(['config', 'jquery', 'bus', 'basic_chart/parameter_select', 'basic_chart/plugins/time_navigator', 'basic_chart/model/basic_instance', 'basic_chart/tools', 'basic_chart/plugins/fail', './widget', 'basic_chart/basic_chart', 'bus', 'container', 'chart/plugins/title_overlay', './basic_chart.css'], function(ECMWF, $, Bus, ParameterSelect, TimeNavigator, BasicInstance, Tools, Fail, DashboardWidget){

var options = {
	preview_resize: false
};

var chart_options = {
	priority: 'normal', editable: false, show_latlong: false, show_title_overlay: true,
	plugins: [
		{mixin: Fail}
	]
};

var product_select_handler;
var projection_select_handler;

$.widget('ui.basicChartDashboardWidget', $.ui.dashboardWidget, DashboardWidget.register('basicChartDashboardWidget', {

	factory: {
		title : 'New basic chart widget',
		kind  : 'basic_chart',
		is_private: !__DEV__,
		data  : { product: 'public_plots' }
	},

	options: {
		data: { },
		has_help: false,
		fixed_size: false,
		editable: false,
		has_external: false,
		messageBus: null // the dashboard bus
	},

	_init: function() {
		var self = this, o = this.options;
		if(typeof o.data.product === 'string') o.data.product = {product: o.data.product}

		$.ui.dashboardWidget.prototype._init.call(this);
		self.element.addClass('basicChartDashboardWidget');

		self.chartBus = self.element.messageBus(o.messageBus); // used by parent class
		var chart_bus = self.content.messageBus(); // the basicChart widget always uses the bus of its dom element, unlike the Chart widget. TODO make it consistent
		var save = false;

		self.old_basetime_requested = false;
		self._height = 64;

		if(!o.data.projection){
			o.data.projection = $.getUserSetting('dashboard-chart-projection', 'europe');
			save = true;
		}

		self._product = BasicInstance.get_instance($.extend(o.data.product, {
			viewport: {width: self._chart_width(), thumbnail: true},
			in_dashboard: true // TODO can remove?
		}));

		if(__DEV__ && !o.active) console.log('instantiating chart for non-active widget...');

		self._chart = self.content
			.basicChart($.extend(chart_options, {
				instance: self._product,
				active: false
				//max_width: width, // pending proper Product support for multiple sizes
			}))
			.bind('resize', function() {
				var width = self._chart_width();
				if(!self._product.data.viewport || (Math.abs(width - self._product.data.viewport.width) > 2))
					self._product.set_size({width: width, thumbnail: true});
			})
			.data('ui-basicChart');

		/*
		bus
			.listen('chart-content-update', function(_, msg) {
				if(msg.changed.get('all')){
					//console.log('chart.save: ', msg.product.save());
					if(this._false){
						o.data.product = msg.product.save();
						if(!self.editing){
							self._emit_changed();
						}
					}
				}
			})
			.listen('chart-product-change', function(){
				self._notify_content_changed();
			})
			.listen('chart-product-availability', function(_, avail) {
				//console.log("BEST BASE TIME ", avail.best_base_time, " OLD CYCLE ", avail.old_cycle);
				if (avail.old_cycle && !self.old_basetime_requested && $.getUserPreference('dashboard_autoupdate')) {
					console.log('dashboardChartWidget.on_availability: auto_update enabled. updating...');
					self.bring_up_to_date();
				}
				else {
					self.out_of_date(avail.old_cycle);
				}
		*/

				//TODO clicking on disabled buttons is propagated to the chart in MSIE

				//TODO setting button state stops the tooltips from working (mouseover event no longer received)
				//     (it looks like the button is destroyed and recreated? though the tooltips are not destroyed).
				//     Tooltip is re-applied as a temporary workaround.
				/*
				self.content.find('.ui-icon-carat-1-w'         ).parent()
					.modeTooltip('destroy')
					.button(avail.current.previous(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Previous time step'});
				self.content.find('.ui-icon-carat-1-e'         ).parent()
					.modeTooltip('destroy')
					.button(avail.current.next(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Next time step'});
				self.content.find('.ui-icon-arrowthickstop-1-w').parent()
					.modeTooltip('destroy')
					.button(avail.current.previous(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'First time step'});
				self.content.find('.ui-icon-arrowthickstop-1-e').parent()
					.modeTooltip('destroy')
					.button(avail.current.next(avail.valid_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Last time step'});
				self.content.find('.ui-icon-arrowthick-1-n'    ).parent()
					.modeTooltip('destroy')
					.button(avail.old_cycle ? 'enable' : 'disable')
					.modeTooltip({content: 'Next base time'});
				self.content.find('.ui-icon-arrowthick-1-s'    ).parent()
					.modeTooltip('destroy')
					.button(avail.current.previousBaseTime(avail.base_time.values) ? 'enable' : 'disable')
					.modeTooltip({content: 'Previous base time'});
				*/
		/*
			})
			.listen('start-animation', function() {
				if(o.active) self.content.chart('animator').start();
			})
			.listen('stop-animation', function() {
				if(o.active) self.content.chart('animator').stop();
			});
		*/

		chart_bus
			.listen_once('chart-content-update', function(_, msg) {
				self._product.events.bind('parameters-changed', function(changed) {
					$.each(changed, function(i) {
						var v = self._product.variables[i];
						v && v.prev && self._emit_changed();
					});
				});
			})
			.listen_once('chart-product-availability', function(_, msg){
				if(msg.valid_time.values.length > 1){
					self.content
						.thumbnailTools({
							selector: '> img, .animation-frames',
							on_create: set_button_states
						})
						.thumbnailTools('add', [
							['ui-icon-play',               function(){ chart_bus.send('tool-animation-toggle');          }, 'Animate'],
							['ui-icon-pause',              function(){ chart_bus.send('tool-animation-pause');           }, 'Pause animation'],
							['ui-icon-stop',               function(){ chart_bus.send('tool-animation-stop');            }, 'Stop animation'],
							['ui-icon-carat-1-w',          function(){ self._product.variables.time.previous();          }, 'Previous time step'],
							['ui-icon-carat-1-e',          function(){ self._product.variables.time.next();              }, 'Next time step'],
							['ui-icon-seek-prev',          function(){ self._product.variables.time.previous_day();      }, 'Previous day'],
							['ui-icon-seek-next',          function(){ self._product.variables.time.next_day();          }, 'Next day'],
							['ui-icon-arrowthickstop-1-w', function(){ self._product.variables.time.first_step();        }, 'First time step'],
							['ui-icon-arrowthickstop-1-e', function(){ self._product.variables.time.last_step()          }, 'Last time step'],
							['ui-icon-arrowthick-1-s',     function(){ self.old_basetime_requested = true;
							                                           self._product.variables.time.previous_basetime(); }, 'Previous base time'],
							['ui-icon-arrowthick-1-n',     function(){ self._product.variables.time.next_basetime();     }, 'Next base time']
						])
				}

				function set_button_states(time) {
					time = self._product.variables.time.value
					var index = msg.base_time.values.reduce(function(acc, val, i){
						return (val.base_time.getTime() === time.base_time.getTime()) ? i : acc
					}, -1)

					var states = [
						['.ui-icon-arrowthick-1-s', index === 0],                              // first bt
						['.ui-icon-arrowthick-1-n', index === msg.base_time.values.length - 1] // last bt
					]
					for(var s in states)
						self.content.find(states[s][0]).parent().button(states[s][1] ? 'disable' : 'enable')
				}

				self._product.base_time && self._product.base_time.subscribe(set_button_states)
			})
			.listen('size-changed', function(_, size) {
				self._update_height();
			});

		// TODO need to activate here for newly added widgets, but for loading of existing dashboards, this causes activation to be done twice
		self.activate();

		if(save){
			setTimeout(function() {
				self._emit_changed();
			},500);
		}
	},

	bring_up_to_date: function() {
		this.content.chart('showLatestCycle');
	},

	activate: function() {
		var self = this;
		console.log('dashboard.basicChart.activate');

		$.ui.dashboardWidget.prototype.activate.call(this);

		self._chart.product_count || self._chart.activate();
		self._product.set_size({width: self._chart_width(), thumbnail: true});
		self.content.css({height: ''}); // TODO this allows the charts to be visible when switching tabs but may not be the best way.

		setTimeout(function() { // TODO
			setTimeout(self._bind(self._update_height), 500);
		}, 1000);
	},

	deactivate: function() {
		var self = this, o = this.options;
		console.log('dashboard.basicChart.deactivate');

		$.ui.dashboardWidget.prototype.deactivate.call(this);

		self._chart && self._chart.animator && self._chart.animator().stop();
	},

	destroy: function() {
		/* enabling this increases memory usage in MSIE
		this.chartBus
			.forget('chart-content-update')
			.forget('chart-product-availability')
			.forget('start-animation')
			.forget('stop-animation');
		*/

		this.element.removeClass('chartDashboardWidget');
		this.content.chart('destroy');
		this.chartBus.destroy();
		//FIXME destroyed busses do not get removed.
		this.chartBus = null;

		this._product && this._product.base_time && this._product.base_time.unsubscribe(this);

		$.ui.dashboardWidget.prototype.destroy.call(this);
	},

	edit: function(enable, form) {
		var self = this;
		console.log('chart.edit', (enable !== false));

		$.ui.dashboardWidget.prototype.edit.call(this, enable, form);

		self.content.chart('animator').stop;
		self.content.thumbnailTools('enable', enable === false)
	},

	_has_time_navigator: function() {
		var has_time_navigator = false;
		this._product.availability(this._product.variables.time.value, function(avail){
			if(avail.valid_time.values.length > 1) has_time_navigator = true;
		});
		return has_time_navigator;
	},

	_on_preview_open: function(el, height, width) {
		var self = this, o = this.options;

		var bus = Bus(el);

		o.messageBus.send('tool-animation-stop');

		self._chart.deactivate();

		var plot_height = height - 10;
		$.each($.extend({
				h3: 34,
				controls: 44
			},
			self._has_time_navigator() ? {tn: 40} : undefined
		), function(i, v){ plot_height -= v; });

		// backend does not currently respect height property, only width
		self._product.set_size(options.preview_resize ? {width: plot_height * self.content_aspect_ratio()} : {});
		self._product.options.priority = 'very-high';

		var controls = el.div('.chart-controls').div('.www-chart-parameter-bar');
		el.find('.chart-controls').div('.bar-2');

		self._chart_preview = {
			bus: bus,
			chart: el.div()
				.basicChart({
					instance: self._product,
					plugins: [
						{mixin: ParameterSelect, options: {controls: controls}},
						{mixin: Tools, options: {parent: '.chart-controls .bar-2'}},
						{mixin: TimeNavigator, options: {element: el.div('#time-navigator')}},
						{mixin: Fail}
					],
					initially_hidden: true
				})
		};
	},

	_on_preview_close: function () {
		var self = this;
		console.log('widgets.basicChart.on_preview_close: ', self._chart_preview);

		self._product.set_size({width: self._chart_width(), thumbnail: true});
		self._product.options.priority = 'normal';
		self._chart_preview.chart.data('uiBasicChart') && self._chart_preview.chart.basicChart('destroy');

		var bus = self._chart_preview.bus;
		if(bus.listeners.listeners.length){
			console.warn('widgets.basicChart._on_preview_close: bus still has listeners. applying workaround...');
			bus.forgetAll();
		}

		self._chart.activate();
	},

	view_in_catalogue: function() {
		console.log('widgets.basic.view_in_catalogue: original_path=', this.options.original_path);

		window.location.href = this.options.original_path
			? ECMWF.fix_host.to_frontend(window.location.protocol + '//' + window.location.host)
					+ this.options.original_path + '?' + $.map(this._product.variables, function(v) { return v.name + '=' + v.to_string(); }).join('&')
			: this._product.canonical_url();
		return this;
	},

	content_aspect_ratio: function() {
		var product = this._chart.product, plot;
		if(product.plots){
			if((plot = product.plots[Object.keys(product.plots)[0]]) && plot.width && plot.height)
				return plot.width / plot.height;
		}
	},

	serialize: function() {
		return $.extend(true,
			$.ui.dashboardWidget.prototype.serialize.call(this),
			{data: {product: {values: this._product.as_json().product.values}}}
		);
	},

	get_state: function(state) {
		var self = this;
		return self.content.chart('getState', true);
	},

	set_state: function(state) {
		var self = this;
		console.log('chart.STATE', state);
		self.content.chart('setState', state);
	},

	get_bus: function() {
		return this.chartBus;
	},

	share: function () {
		var o = this.options;
		Bus().send('product-send', {type: 'chart', product: o.data.product.name});
		return this;
	},

	on_show: function() {
		this.content.resize();
		return this;
	},

	_update_height: function() {
		// the chart height is not set here because that is done by the basicChart widget.
		var self = this, o = this.options;

		if(!o.collapsed){
			var width = self.element.width();
			var new_height = self.content.height();
			if(width){
				if(new_height !== self._height){
					self._height = new_height;

					self.content.triggerHandler('resize', {size: {height: new_height}});

					this.get_bus().send('height-changed', this.widget_height());
				}
			}
		}
		return self;
	},

	// the height may be not be set. must subscribe to height-changed to get the real height.
	widget_height: function() {
		var self = this, o = self.options;

		var border_height = 2;
		var header_height = 24;

		if(!self._height) console.log('widgets.basicChart: _height not set');

		return (o.collapsed ? 0 : self._height) + border_height + header_height;
	},

	_chart_width: function() {
		return this.content.width();
	},

	/*
	widget_height_async: function () {
		var self = this;

		var border_height = 2;
		var header_height = 24;

		return $.Deferred(function(dfd){
			setTimeout(function(){
				var height = (self._height || self.content.height()) + border_height + header_height;
				dfd.resolve(height);
			}, 2000);
		}).promise();
	},
	*/

	content_height: function() {
		var self = this;

		return self.content.height();
	}
}));

});
