define(['config', 'jquery', 'bus', 'ajax_queue', 'application', './reporter', 'chart/plugins/resize', 'core/settings', 'unit', 'thumbnail-list', 'menu_pane', 'messages', 'data-availability', 'chart', 'layer-list', 'chart/plugins/save', 'toolbar', 'time_navigator', 'product_manager', 'shortcuts', 'tooltip/tipsy', 'meteograms', 'focusable', 'ajax-indicator', 'chart/latlon', 'legends', 'animation-controls', 'cities', './time_series', './big_title', './forecaster.css', './application.css', './gadget.css', 'tool-buttons/clear', 'tool-buttons/fullmap', 'tool-buttons/undo', 'tool-buttons/redo', 'tool-buttons/zoomin', 'tool-buttons/zoomout', 'tool-buttons/drag_pan', 'tool-buttons/drag_zoom', 'tool-buttons/select', 'tool-buttons/vertical', 'tool-buttons/goto_marker', 'tool-buttons/print', 'tool-buttons/layers'], function(ECMWF, $, Bus, Ajaxqueue, Application, Reporter, resize) {

	new Application(function (application) {

		ECMWF.browser.supports_svg_img(function (supported){
			if(!supported) {
				console.warn('svg-image NOT supported');
				// see widgets/chart/plugins/svg_image.js for better implementation
				if($.getUserPreference('image_format') === 'svg'){
					ECMWF.alert({
						title:   'SVG Not Supported',
						content: '<b>This browser does not support SVG images.</b><p style="margin-top:1em;">Please go to Help / Preferences and change Image Format to PNG.</p>'
					});
				}
			}
		});

		if (!ECMWF.request.kiosk) {
			$.fn.keyboard = function () {
				return this;
			}
		}

		window.Workspaces && Workspaces.init();

		var product_thumbnail_dialogue_options = {
			text: 'Select from these ECMWF Products to add to your personal list',
			actions: [{
				name: 'Add to product list',
				on_click: function (item) {
					//$('#products').thumbnailDialogue('hideByName', item);
					Bus().send('product-dialogue-selection', {
						names: [item]
					});
				}
			},
			{
				name: 'View in map',
				on_click: function (item) {
					Bus().send('product-thumbnail-click', {
						name: item
					});
				}
			}],
			hide_user_items: true
		};

		var projection_thumbnail_dialogue_options = {
			text: 'Select from these ECMWF Projections to add to your personal list',
			kind: 'projection',
			actions: [{
				name: 'Add to projection list',
				on_click: function (item) {
					//$('#projections').thumbnailDialogue('hideByName', item);
					Bus().send('projection-dialogue-selection', {
						names: [item]
					});
				}
			},
			{
				name: 'Apply to map',
				on_click: function (item) {
					Bus().send('projection-thumbnail-click', {
						name: item
					});
				}
			}],
			hide_user_items: true
		};

		$('#products').thumbnailList({
			kind: 'product',
			visible_items: 6,
			show_add_button: function () {
				$('#products').thumbnailDialogue(product_thumbnail_dialogue_options);
			}
		});
		$('#projections').thumbnailList({
			kind: 'projection',
			visible_items: 6,
			show_add_button: function () {
				$('#projections').thumbnailDialogue(projection_thumbnail_dialogue_options);
			}
		});

		$('#views').menuList();
		$('#save').menuList();
		$('#test').menuList();
		$('#go').menuList().find('li:first').removeClass('enabled').addClass('disabled');
		$('#messages').messages();
		//$("#timers").timersInfo();
		//$("#notifications").notifications({ counter: $("#notifications-counter")});
		$('#availability').dataAvailability({
			counter: $('#availability-counter')
		});

		// Create all menus
		var menus = ['products', 'projections', 'views', 'save', 'availability', 'test', 'go'];
		for (var i in menus) {
			$('#' + menus[i] + '-menu').button({icons: {secondary: 'ui-icon-triangle-1-s'}});

			$('#' + menus[i] + '-pane').menuPane({
				group: '#menu-panes > div',
				owner: $('#' + menus[i] + '-menu')
			});

			// When a thumbnail is selected, close the menu
			(function (menu) {
				$('#' + menu).bind('select', function () {
					$('#' + menu + "-pane").trigger('close');
					return false;
				});
			})(menus[i]);

		}

		$.fn.workspaceMenu && $('#menu-bar').workspaceMenu({menu: new ECMWF.MenuBar('#menu-bar', 'menu-panes'), position: 5});

      load_chart(function(_chart){
			chart = _chart;

			(chart && chart.create()) || ECMWF.alert({
				title: 'Chart not found',
				content: (
					ECMWF.request.widget
					? 'Cannot find widget "<b>' + $('<span>').text(ECMWF.request.widget).text() + '</b>"'
					: 'Chart not found: ' + $('#chart').chart('option', 'product')
				)
			});
		});

		if($('body').hasClass('html')){
			var fs = {
				button: $('<div class="fullscreen-button">').button({icons: {primary: 'ui-icon-arrow-4-diag'}, text: false, label: 'Full size'}).prependTo('#gadget-head')
					.click(function(){
						$('.sidebars, #content-heading, .header-bar').toggle();
						fs.handlers[fs.i++ % 2]();
						$('body').triggerHandler('layout');
					}),
				handlers: [
					function() {
						$('#main').addClass('fullsize');
						//fs.button('option', 'icons', {primary: 'ui-icon-arrowthick-2-ne-sw'});
						fs.button.button('option', 'icons', {primary: 'ui-icon-arrow-2-ne-sw'});
					},
					function() {
						console.log('2');
						$('#main').removeClass('fullsize');
						fs.button.button('option', 'icons', {primary: 'ui-icon-arrow-4-diag'});
					}
				],
				i: 0
			}
		}

		$('#toolbar').toolbar({
			tools: [
				['clear', 'full-map'],
				['undo', 'redo'],
				['zoom-in', 'zoom-out'],
				['drag-pan', 'drag-zoom', 'select', 'vertical'],
				['goto-marker'],
				['print']],
			tooltips: true
		});

		$('#toolbar-right').toolbar({
			tools: [
				['layers']
			],
			tooltips: true
		});

		// Add layer list
		$('#layer-list').layerList();
		$('#layer-list-pane').menuPane({
			resize: true,
			align: 'right',
			owner: $('.button.layers')
		});

		// Time-slider
		$('#time-navigator').timeNavigator().hide();

		$(document).productManager().shortcuts({
			shortcuts: [{
				key: $.ui.keyCode.UP,
				signal: 'next-base-time'
			},
			{
				key: $.ui.keyCode.DOWN,
				signal: 'prev-base-time'
			},
			{
				key: $.ui.keyCode.LEFT,
				signal: 'prev-step'
			},
			{
				key: $.ui.keyCode.RIGHT,
				signal: 'next-step'
			},
			{
				key: $.ui.keyCode.HOME,
				signal: 'first-step'
			},
			{
				key: $.ui.keyCode.END,
				signal: 'last-step'
			},
			{
				key: $.ui.keyCode.ESCAPE,
				signal: 'cancel'
			},
			{
				key: 52, // '4'
				callback: function () {
					$('#products').thumbnailDialogue(product_thumbnail_dialogue_options);
				}
			},
			{
				key: 53, // '5'
				callback: function () {
					$('#projections').thumbnailDialogue(projection_thumbnail_dialogue_options);
				}
			},
			{
				key: 54, // '6'
				signal: 'open-layer-thumbnail-dialogue'
			},
			{
				key: $.ui.keyCode.NUMPAD_ADD,
				signal: 'tool-zoom-in'
			},
			{
				key: $.ui.keyCode.NUMPAD_SUBTRACT,
				signal: 'tool-zoom-out'
			},
			{
				key: $.ui.keyCode.COMMA,
				signal: 'animation-speed-down'
			},
			{
				key: $.ui.keyCode.PERIOD,
				signal: 'animation-speed-up'
			},
			{
				key: 51, // '#' key
				callback: function() { // '3'
					require(['chart/plugins/timers'], function(plugin) {
						var chart = $.fn.chart && $('.chart').chart('instance') || ($.fn.basicChart && $('.basic').basicChart('instance'));
						if(chart && chart.add_plugin('Timers', plugin)){
							require(['notify'], function() {
								$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Calltree timers are now enabled<br/>Subsequent requests will have timer information available.'});
							});
						}else{
							Bus().send('open-calltree');
						}
					});
				}
			},
			{
				key: 61, // '=' key
				signal: 'show-timers'
			},
			{
				key: 187, // '=' key (webkit)
				signal: 'show-timers'
			},
			{
				key: 70, // 'f'
				signal: 'fullscreen'
			},
			{
				key: 80, // 'p'
				signal: 'presentation'
			},
			{
				key: 85, // 'u'
				signal: 'undo-key-press'
			},
			{
				key: 82, // 'r'
				signal: 'redo-key-press'
			}]
		});

		// Make all gadgets
		/* $(".product-gadget").gadget(); */

		Reporter();

		Bus()
			.listen('multigrams-list-changed', function (_, msg) {
				$.setUserSetting('meteograms-list', msg);
				$.logMessage('Meteogram List updated');
			})
			.listen('menu-help-help-select', function (_, msg) {
				window.open(ECMWF.options.settings.eccharts_help);
			})
			.listen('menu-help-preferences-select', function (_, msg) {
				require('preferences') && $(window).preferences();
			})
			.listen('menu-go-dashboard-select', function (_, msg) {
				location.href = ECMWF.options.ajax.dashboard.dashboard;
			})
			.listen('chart-content-update', function (_, msg) {
				msg.changed.get('all') && chart.save(msg);
			})
			.listen('dialogue-state-changed', function (_, msg) {
				var pref = $.getUserSetting('dialogue-states', {});
				pref[msg.name] = msg;
				$.setUserSetting('dialogue-states', pref);
			})
			.listen('latlon-location-select', function (event, msg) {
				if(msg.from !== 'init'){
					$.setUserSetting('last-probe-location', msg);
				}
				$.logMessage($.formatGeoLocation(msg) + ' ' + (msg.title || ''));
			})
			.listen('presentation', function () {
				$('.no-presentation').toggle();
				$('body').triggerHandler('layout');
			})
			.listen('workspace-select', function (_, workspace) {
				$.logMessage('Switched to workspace: ' + workspace.title);
				$('#chart').chart('setState', $.getUserSetting('last-product'));
			});

		var loc = $.getUserSetting('last-probe-location', null);
		if (loc) {
			setTimeout(function () {
				loc.from = 'init';
				Bus().send('latlon-location-select', loc);
			},
			500);
		}

		var states = $.getUserSetting('dialogue-states', {});

		$('body').animationControls({state: states['controls']});

		var meteogram_settings = $.getUserSetting('meteograms-list', {
			'products': ['10d_epsgram_2t']
		});

		// on-demand dialogue instantiation
		$.each({
				probe:         { title: 'Probe'},
				meteograms:    { title: 'Meteograms', options: meteogram_settings},
				legends:       { options: {show_zoom_buttons: true}},
				cities:        { title: 'City Finder'},
				info:          { widget: 'bigTitle'},
				'time-series': { title: 'Time Series', widget: 'chartTimeSeries'}
			},
			$.viewDialogue
		);

		$('body').container({
			layout: 'border',
			north: $.extend($('.header-bar, #content-heading, #menu-bar, #page-wrapper > header'), {autoHeight: true}), // TODO move template specifics to config
			center: $(ECMWF.options.settings.content || '#content'),
			south: $('#status-bar')
		});

		$('#gadget').container({
			layout: 'border',
			north: $('#gadget-head'),
			center: $('#gadget-content'),
			south: $('#time-navigator')
		});

		$.fn.focusable();

		$('#ajax-loader').ajaxIndicator();

		$('#gadget-content').bind('resize.startup', function resize_startup(_, a){
			if(a && a.height/* && a.height > 0*/){
				setTimeout(function(){
					$('#time-navigator').show().animate({opacity: 1}, function(){
						$('#time-navigator').find('> .time-nav').triggerHandler('layout');
					});
				}, 500);
				$('#gadget-content').unbind('resize.startup');
			}
		});

		var chart;
		function load_chart(done){

			if (ECMWF.request.widget) {

				Ajaxqueue({
					url: ECMWF.options.ajax.backend.get_widget,
					json: {name: ECMWF.request.widget},
					success: function (result) {
						done(new dashboardChartDelegate(result));
					},
					error: function (result) {
					}
				});
			}
			else if (ECMWF.request.product) {
				done(new presetChartDelegate());
			} else {
				done(new chartDelegate());
			}
		}

		function chartDelegate() {

			this.options = {
				projection: (ECMWF.request.projection || 'europe'),
				use_outline: true,
				plugins: [resize]
			};

			this.save = function (msg) {
				$.setUserSetting('last-product', msg.product.get_save_state());
			}

			this.create = function () {
				this.options.product = $.getUserSetting('last-product', 'wind');
				return $('#chart').chart($.extend(this.options, {
					title: $('#chart-title')
				}));
			}
		}

		function presetChartDelegate() {

			this.options = {
				product: ECMWF.request.product,
				projection: (ECMWF.request.projection || 'europe'),
				plugins: [resize]
			};

			this.save = $.noop; // not savable

			this.create = function () {
				console.log('presetChartDelegate: product=', this.options.product);
				return $('#chart').chart($.extend(this.options, {
					title: $('#chart-title')
				}));
			}
		}

		function dashboardChartDelegate(widget) {

			ECMWF.request.widget = widget;

			this._title = widget.title;
			console.log('dashboardChartDelegate: widget.data:');
			console.log(widget.data);

			this.options = {
				product: widget.data.product,
				projection: widget.data.projection,
				plugins: [resize]
			};

			this.save = function (msg) {
				if ($.getUserSetting('auto-save-widgets', false)) {
					widget.data.product = msg.product.save();
					Ajaxqueue({
						priority: 'very-high',
						url: ECMWF.options.ajax.backend.save_widget,
						json: widget,
						logErrors: 'Save product',
						success: function (result) {
							console.log('done. ', result);
						}
					});
				}
			}

			this.create = function () {
				console.log('dashboardChartDelegate.create: product=', this.options.product);
				var c = $('#chart').chart(this.options);
				$('#gadget').triggerHandler('layout');
				return c;
			}
		}
	});

});
