define(['config', 'jquery', 'ajax_queue', 'bus', 'chart/layer', 'element', './time_series.css'], function(ECMWF, $, Ajaxqueue, Bus, Layer, Dom) {

	var count = 0;

	$.widget('ui.chartTimeSeries', {

		options: {
			width: 500,
			height: 100,
			messageBus: Bus()
		},

		_init: function() {
			var self = this,
			o = this.options;

			count++;

			self.name = 'time_series_' + count;
			self.location = $.getUserPreference('default_location');

			self.element
				.bind('show', function() {
					$.openProbe(self);
				})
				.bind('hide', function() {
					$.closeProbe(self);
				});

			self.element.addClass('time-series drop-shadow-small');
			self.loc = $('<div>').appendTo(self.element).attr('id', 'location');
			self.content = $('<div>').appendTo(self.element);

			self.table = $('<ul>').appendTo(self.content);
			self.tr = [];
			self.img = [];
			self.title = [];

			self.element.bind('show', function() {
				self._update();
			});

			o.messageBus.listen('chart-time-select', function (event, msg) {
				if (self.time && ((self.time.base_time - msg.base_time) !== 0)) {
					self.time = msg;
					self._update();
				}
			}).listen('chart-content-update', function (event, msg) {
				if ((self.product !== msg.product) || (self.time && msg.time && ((self.time.base_time - msg.time.base_time) !== 0))) {
					self.product = msg.product;
					self.time = msg.time || self.time;
					self._update();
				}
			}).listen('chart-product-change', function (event, msg) {
				self.product = msg.product;
				self.time = msg.time || self.time;
				self._update();
			}).listen('latlon-location-select', function (event, msg) {
				self.location = msg;
				self._update();
			}).listen('chart-product-new-availability', function(event, msg){
				//the widget doesnt need to know about the availability, only that it has changed.
				console.log('timeSeries: NEW AVAILABILITY - TESTME', msg);
				self._update();
			});

			self._update(); // Ask for a resize
		},

		_reset: function (txt) {
			this.loc.html(txt || '<div>Click on the map with the probe tool:</div><ul class="ec-toolbar"><li class="sprite-outer"><div class="button select sprite">');
		},

		_update: function () {

			var self = this, o = this.options;

			if (!self.location || !self.product || !self.time || !(self.element.is(":visible"))) {
				self._reset();
				self.element.resize();
				return;
			}

			var title = 'Getting values for location ' + $.formatGeoLocation(self.location);
			if (self.location.title) {
				title += ', ' + self.location.title;
			}
			self.loc.html(title);

			//self.resetCurve()
			Ajaxqueue.cancelByTag(self.name);

			function ensure(j) {
				if (!self.tr[j]) {

					self.title[j] = self.table.li('-');

					self.tr[j] = self.table.li();

					self.img[j] = $('<img>').appendTo(self.tr[j])
						.css({width: o.width, height: o.height})
						.attr({src: ECMWF.options.files.empty_image});
				}
			}

			var layers = self.product.layerList().reverse();
			var visible = false;

			$.each(layers, function (j, layer) {

				ensure(j);

				if (layer.has_probe) {

					self.title[j].html(new Layer(layer).get_title()).show();
					self.img[j].attr({
						src: ECMWF.options.files.empty_image
					});
					self.tr[j].show();

					Ajaxqueue({
						url: ECMWF.options.ajax.backend.get_time_series,
						//cancelled_by: self.product.owner,
						json: {
							product: self.product.as_json(null, self.time),
							location: self.location,
							layer: layer.id
						},
						priority: 'low',
						status: ECMWF.ajax_loader(self.tr[j]),
						img: self.img[j],
						tag: self.name,
						logErrors: 'Time Series',
						timeout: 5 * 60 * 1000,

						success: function (json) {

							var title = 'Data values near location ' + $.formatGeoLocation(json);
							if (json.title) {
								title += ", " + json.title;
							}
							self.loc.html(title);

							if (json.units) {
								var symbol = json.units.html_symbol || json.units.name;
								self.title[j].html(new Layer(layer).get_title()  + ' (' + symbol + ')');
							}

						}
					});
							visible = true;
				}
				else {
					self.tr[j].hide();
					self.title[j].hide();
				}

			});

			for (var i = layers.length; i < self.tr.length; i++) {
				self.tr[i].hide();
				self.title[i].hide();
			}

			if (!visible) {
				ensure(0);
				self.title[0].html('No visible data layer').show();
			}

		},

		destroy: function () {
			var self = this,
			o = this.options;

			self.element.removeClass('ui-widget ui-widget-content ui-corner-all time-series');

			// Cleanup CSS and event handling
			// self.element.removeClass("....").unbind("....") 
			// Destroy created DOM elements
			self.element.empty();

			return this;
		}
	});
});
