define(['config', 'jquery', 'ajax_queue', 'bus', 'chart/layer', 'jquery_ui/widget', './legends.css'], function(ECMWF, $, Ajaxqueue, Bus, Layer) {

    var count = 0;

    $.widget('ui.legends', {

        options: {
            show_zoom_buttons: false,
            messageBus: Bus()
        },

        _init: function(){
            var self = this, o = this.options;

            count++;

            self.name = 'legends_' + count;

            self.factor = $.getUserSetting('legends_size', 1);

            self.element.addClass('legends');

            self.size = {
                width: 350 * self.factor,
                height: 50 * self.factor
            }
            
            var buttons = self.element.div().div('.buttons');

            if(o.show_zoom_buttons){
                buttons.span('.ui-icon ui-icon-zoomin')
                    .css({top: 0, left: 0})
                    .click(function() {
                        self.wrapper.css('font-size', function(_, old) {
                            self.size.width *= 1.2; self.size.height *= 1.2;
                            self._update(true);
                            self.factor *= 1.2;
                            setTimeout(function() { $.setUserSetting('legends_size', self.factor); }, 10);
                            return parseFloat(old)*1.2;
                        });
                    });

                buttons.span('.ui-icon ui-icon-zoomout')
                    .css({top: 0, left: 16})
                    .click(function() {
                        self.wrapper.css('font-size', function(_, old) {
                            self.size.width /= 1.2; self.size.height /= 1.2;
                            self._update(true);
                            self.factor /= 1.2;
                            setTimeout(function() { $.setUserSetting('legends_size', self.factor); }, 10);
                            return parseFloat(old)/1.2;
                        });
                    });
            }

            self.element.bind('show', function() {
                self._update();
            });

            self.wrapper = self.element.div('.legends-wrapper');
            self.ids     = [];
            self.ul      = self.wrapper.div('.layers').ul();
            self.tr      = [];
            self.img     = [];
            self.title   = [];

            o.messageBus
                .listen('chart-time-select', function(event, msg) {
                    self.time = msg;
                    /* self._update(); */
                })
                .listen('chart-content-update', function(event, msg) {
                    self.time = msg.time || self.time;
                    self._layers(msg);
                })
                .listen('latlon-location-select', function(event, msg) {
                })
                .listen('layers-visibility-change', function(event, msg) {
                    self._layers(msg);
                })
                .listen('chart-product-change', function(event, msg) {
                    //self.ids = [];
                    self._layers(msg);
                });

            if(self.factor != 1){
                self.wrapper.css('font-size', function(_, old) {
                    return parseFloat(old) * self.factor;
                });
            }
            
            self._update();
        },

        _layers: function(msg) {
            var self = this, o = this.options;

            self.product = msg.product;
            self._update();
         },

        _update: function(force){
            var self = this, o = this.options;

            if (!self.time || !(self.element.is(":visible"))) {
                self.element.resize();
                return;
            }

            var layers = self.product.layerList().reverse();

            var ids = [];
            $.each(layers , function(j, layer) {
                ids.push($.makeCacheKey(layer));
            });

            if(ids.join(":") === self.ids.join(":") && !force)
            {
                // No change
                return;
            }

            self.ids = ids;

            function ensure(j) {
                if(!self.tr[j]) {

                    var li = self.ul.li();

                    self.title[j] = li.h4('-');

                    self.tr[j]  = li.div();

                    self.img[j] = self.tr[j]
                        .img({src: ECMWF.options.files.empty_image})
                        .css(self.size);
                }
            }

            var visible = false;

            $.each(layers , function(j, layer) {
                    
                ensure(j);

                if(layer.has_data && layer.visible) {
                    Ajaxqueue({
                            cancelled_by: self.product.owner,
                            url         : '/backend/layer-legend/',
                            priority    : 'low',
                            status      : ECMWF.ajax_loader(self.tr[j]),
                            cache       : 24*60*60,
                            json        : {product: self.product.as_json({width: self.width, height: self.height}, null), layer: layer.id, size: self.size},
                            img         : self.img[j],
                            success     : function() {
                                self.img[j].css({width: 'auto', height: 'auto'});
                            }
                        });

                        self.title[j].html(new Layer(layer).get_title()).parent().show();
                        visible = true;
                    }
                else {
                    self.tr[j].parent().hide();
                }
            });

            for(var i = layers.length; i < self.tr.length; i++) {
                self.tr[i].parent().hide();
            }

            if(!visible) {
                ensure(0);
                self.img[0].attr({src: ECMWF.options.files.empty_image});
                self.title[0].html('No visible data layer').parent().show();
            }
        },

        destroy: function(){
            var self = this, o = this.options;

            self.element.removeClass('legends');

            // Cleanup CSS and event handling
            // self.element.removeClass("....").unbind("....") 
            // Destroy created DOM elements
            self.element.empty();

            return this;
        }
    });
});
