
define(['jquery', 'bus', 'test-lib', 'grid/testable', 'chart_app/router', 'filterable-list', 'grid/products', 'user_info'], function($, Bus, Tests, GridTestable, router, list, Products, User){

	var dont_run = [
		//'Test cache'
	]

	var tests = new Tests([
		{
			name: 'Test login',
			start: function(){
				this.assert_no_element('a[href="/auth/login"]').finished();
			}
		},
		/* TODO this test requires the ability to change package
		{
			name: 'Test interactive charts',
			start: function(){
				var test = Testable.test = this;
				Testable.ensure_interactive_product(function() {
					test.finished();
				});
			}
		},
		*/
		{
			name: 'Test facets',
			start: function(){
				var test = Testable.test = this;
				var facets;
				var min_products = 3;
				var g = new GridTestable('.column-2 .catalogue-grid');
				g.test = test;

				function wait_for_scrollbar(done){
					if(false && !~$.inArray(router._path.dir, ['medium_range', 'mofc_multi'])){
						// scrollbar will be present for packages with many products
						test.wait_for(g.bind(g.has_scrollbar), 'has scrollbar').then(done);
					}else{
						setTimeout(done, 1000);
					}
				}

				test.series([
					function(done){
						test.assert($('.column-1 > .filters h2:first').text() === 'Filters', 'h2: ' + $('.column-1 > .filters h2:first').text());
						test.wait_for('.column-1 .facets', 'facets block').then(function(){
							facets = $('.column-1 .facets');
							facets.find('input[type="checkbox"]:checked').click();
							test.assert(facets.find('input[type="checkbox"]:checked').length === 0, 'checkboxes not cleared');

							test.wait_for(g.bind(g.has_status), 'has status').then(function(){
								console.log('n_products=', g.status_count());
								test.assert(g.status_count() >= min_products, 'not enough products: ' + g.status_count());
								done();
							});
						});
					},
					function(done){
						test.assert(g.el.height() > 80, 'grid height:' + g.el.height() + ' el=' + g.el.attr('class'));
						test.wait_for(g.bind(g.has_thumbnails), 'has thumbnails').then(function(){
							test.wait_for(
								function(){ return g.el.find('.grid-items .thumbnail-wrapper').length >= min_products},
								function(){ return 'not enough thumbnails: ' + g.el.find('.grid-items .thumbnail-wrapper:visible').length})
								.then(function() {
									test.assert(g.el.find('.grid-items .thumbnail-wrapper:visible').length >= min_products, 'not enough visible thumbnails: ' + g.el.find('.grid-items .thumbnail-wrapper:visible').length);
									wait_for_scrollbar(function(){
										select_facet(function(){
											if(!~$.inArray(router._path.dir, ['s2s', 'medium'])){ // first facet of s2s brings 24 products which is not enough to reliably remove the scrollbar
												test.wait_for(g.bind(g.not_has_scrollbar), 'scrollbar hide')
													.then(done);
												}else{
													done()
												}
										});
									});
							});
						});

						function select_facet(done) {
							var facet = facets.find('li li:first');
							var n_facet_products = parseInt(facet.text().replace(/.*\((.*)\)/, '$1'), 10);
							test.assert(n_facet_products > 0, 'no facet counts');
							facet.find('input').click();
							test.wait_for(function(){ return g.status_count() === n_facet_products; }, 'n products=' + g.status_count() + ' expected=' + n_facet_products)
								.then(done);
						}
					},
					function clear_facets(done){
						Testable.clear_facets(done);
					}
				], function(){ console.log('finished'); });
			}
		},
		{
			name: 'Test search',
			start: function(){
				var test = Testable.test = this;
				var g = new GridTestable('.column-2 .catalogue-grid');
				g.test = test;

				test.series([
					function(done) {
						var names = {'chart-catalogue': 'w_t2m30mw_acmad'}
						var product_name = names[router._path.dir] || g.first_product().name();
						$('.thumbnail-list-filter input').val(product_name).trigger('keyup');
						test.wait_for(
							function(){	return g.n_visible_items() === 1 },
							function(){ console.log('visible_items:', g.visible_items()); return 'show single search result: ' + product_name + ' n=' + g.n_visible_items()})
						.then(function(){
							$('.thumbnail-list-filter input').val('').trigger('keyup');
							test.wait_for(
								function(){	return g.n_visible_items() > 1 },
								function(){ return 'clear search: ' +  product_name + ' n_visible=' + g.n_visible_items()})
							.then(done)
						})
					}
				]);
			}
		},
		{
			name: 'Test results menu',
			start: function(){
				var test = Testable.test = this;
				test.set_timeout(10000);

				test.series([
					function(done) {
						Testable.select_product_via_menu(1, done);
					}
				], function(){ console.log('finished'); });
			}
		},
		/* TODO fix the ability to change package
		{
			name: 'Tropical cyclones',
			start: function() {
				var test = this;
				test.series([
					function(done){
						router.set_dir('tcyclone');
						done();
					}
				], function(){ console.log('finished'); });
			}
		}
		*/
		{
			name: 'Test parameter menu',
			start: function() {
				var test = Testable.test = this;
				test.series([
					function(done){ Testable.ensure_product_page(done); },
				], function(){ console.log('finished'); });
			}
		},
		{
			/*
			 *  Test that parameters set for a chart product are preserved when the product is changed.
			 *
			 *  TODO test parameters other than base time.
			 */
			name: 'Test parameters preserved',
			start: function() {
				var test = Testable.test = this;
				test.set_timeout(25000);
				test.series([
					// change base-time and check it is preserved
					function(done){ Testable.ensure_product_page(done); },
					function(done){ Testable.change_base_time(1, done); },
					function(done){ Testable.change_base_time(2, done); }/*,

					// TODO for this to work reliably, we need to pick products that have common parameter values
					function(done){ Testable.change_base_time(3, function(){
						if($('.www-chart-container .basic').length){
							var chart = $('.www-chart-container .basic').basicChart('instance');
							var base_time = chart.product.variables.time.value.base_time;

							Testable.select_product_via_menu(1, function(){
								console.log('checking basic chart...');
								var new_base_time = chart.product.variables.time.value.base_time;
								test.assert(new_base_time.getTime() === base_time.getTime(), 'base time equal: ' + base_time + ' ' + new_base_time);
								done();
							});
						}else{
							console.error('FIXME unhandled chart type');
						}
					})}
					*/
				], function(){ console.log('finished'); });
			}
		},
		{
			name: 'Test text',
			start: function(){
				var test = Testable.test = this;
				test.series([
					function(done){
						Testable.ensure_product_page(function(){
							test.wait_for(function() {
								return $('.chart-column .product-text').length === 1;
							}, 'product text').then(function(){
								test.wait_for(function() {
									return $('.chart-column .product-text').text().length > 20;
								}, 'product text content').then(done)
							});
						});
					}
				]);
			}
		},
		{
			// test that repeated views of the same product are successful.
			name: 'Test cache',
			start: function(){
				var test = Testable.test = this;

				var tests = [
					Testable.clear_facets
				]

				// cycle through several products:
				var n_products = 3
				var n_iterations = 10
				Array.apply(0, Array(n_products * n_iterations)).map(function(_, i) {
					tests.push(function(done) { test.set_timeout(10000); Testable.set_product(i % n_products, done); });
				})

				// check spinner. there can be multiple spinners temporarily while products are changed quickly.
				tests.push(function(done) {
					var chart = $('.basic');
					test.wait_for(function() {
						return chart.find('.spinner').length < 2;
					}, 'single spinner').then(done);
				});

				test.series(tests, function(){ console.log('finished'); });
			}
		},
		{
			name: 'Test meteogram overlay',
			start: function() {
				var test = Testable.test = this;

				switch(router._path.dir){
					case 's2s': // no clickable products, but test needs to pass on cams website
						return test.finished();
				}

				test.series([
					function(done){
						Testable.ensure_clickable_product(function() {
							if(!$('#chart-legends:visible').length) return done(); // non-interactive chart - no meteogram.

							$('.chart-image').click();
							test.wait_for(function() {
								return $('.modal:visible').length;
							}, 'overlay open').then(function() {
								test.wait_for(function() {
										return $('input.lat, input.lon').length === 2;
									}, 'lat lon inputs: expected 2, got ' + $('input.lat, input.lon').length)
									.then(function(){
										$('#location-overlay .close').click();
										test.wait_for(function() {
											return $('#location-overlay:visible').length === 0;
										}, 'meteogram close').then(done);
									});
							});
						});
					}
				], function(){ console.log('test_meteogram finished'); });
			}
		},
		{
			name: 'Test animation',
			start: function() {
				var test = Testable.test = this;
				test.series([
					function(done){
						Testable.ensure_animatable_product(function(){
							Testable.test_animation(done);
						});
					}
				], function(){ console.log('finished'); });
			}
		}
	]).dont_run(dont_run).run();

	var Testable = {
		chart: {
			element: '.www-chart-container, .chart-column .chart'
		},

		ensure_product_page: function(done) {
			var self = this, g = new GridTestable('.column-2 .catalogue-grid'), test = Testable.test;

			if(router._path.page){
				return done()
			}

			// a variation of $.findWhere what assumes that the properties are getter functions instead of plain keys
			function findWhere(list, properties){
				return $.map(list, function(l){
					if($.map(properties, function(v, i){
						if(l[i]() === v) return true;
					}).length) return l;
				})[0];
			}

			var first_product = list.items[list.filtered[0]];
			$('.column-2 .grid-items div.item:visible:first').click();
			var pname = g.first_product().name();
			if(pname !== first_product.name()) console.warn('why is first thumbnail not the first in the filtered list?', pname, product.name);
			var product = findWhere(list.items, {name: pname});
			test.wait_for(
				function() {
					return $(self.chart.element).is(':visible') && $('span:contains(Filter results)').length && ($('h1:first').text() === product.title_());
				},
				function() { return 'product page: ' + product.title_() + ' ---- "' + $('h1:first').text() + '"'}
			).then(done);
		},

		ensure_landing_page: function(done) {
			var self = this, test = Testable.test;
			console.log('%censure_landing_page', 'background-color:#ff9');

			Products.packages.then(function(packages) {
				var pkg = (function get_package(){
					var p = list.items[list.filtered[0]];
					return p.data.package;
				})();

				// TODO fix the products
				if(pkg === 'medium'){
					console.warn('product claims to be in nonexistent medium package');
					pkg = 'catalogue';
				}

				var p = $.findWhere(packages, {name: pkg});
				test.assert(!!p, 'package not found. pkg=' + pkg)
				function is_package_landing(){
//					return !!~$.inArray($('h1:visible').text(), [p.title, 'Graphical products catalogue']);
return !!~$.inArray($('h1:visible').text(), [p.title, 'Graphical products catalogue', 'Charts']);
				}

				if(is_package_landing()) return done();

				var button = $('.chart-column .back-button');
				test.assert(button.length === 1, 'back button');
				button.click();
				test.wait_for(is_package_landing, 'package landing page: ' + p.title).then(done);
			});
		},

		ensure_animatable_product: function(done) {
			var self = this;
			Testable.test._set_timeout_info('animatable product');

			function set_product(name, title){
				var basic = $(self.chart.element).find('.basic');
				if(basic.length){
					var bus = basic.messageBus();
					var message = 'product-swapped-and-loaded'
				}else{
					bus = $(self.chart.element).chart('instance').options.messageBus;
					message = 'chart-content-update'
				}
				bus.clear().listen_once(message, function(){
					have_product();
				});
				router.set_product(name, title);
			}

			function have_product() {
				Testable.test.wait_for('.time-nav .canvas-slider', 'time navigator').then(done);
			}

			switch(router._path.dir){
				case 'catalogue':
					if(router._path.page !== 'medium-2mt-wind30'){
						set_product('medium-2mt-wind30', '2m temperature and 30m winds');
					}else{
						set_product('medium-mslp-wind850', 'Mean sea level pressure and wind speed at 850 hPa');
					}
					break;
				case 'medium':
					if(router._path.page !== 'w_t2m30mw_acmad'){
						set_product('w_t2m30mw_acmad', '2m temperature and 30m winds (ACMAD)');
					}else{
						set_product('w_mslrain_acmad', 'Rainfall and MSLP over Africa (ACMAD)');
					}
					break;
				case 'medium_range':
					have_product();
					break;
				case 's2s':
					if(router._path.page !== 's2s_anomaly_cawcr_2MTM'){
						set_product('s2s_anomaly_cawcr_2MTM', 'BoM 2m Temperature Anomaly');
					}else{
						set_product('s2s_EFI_cawcr_EFI', 'BoM EFI');
					}
					break;
				case 'c3s_seasonal':
					have_product();
					break;
				default:
					Testable.test.assert(false, 'package ' + router._path.dir + ' not supported by test');
					break;
			}
		},

		ensure_product_with_basetime: function(done) {
			var self = this, test = Testable.test;
			test._set_timeout_info('product with base time');

			switch(router._path.dir){
				case 'medium':
					Testable.set_product(router._path.page !== 'pincr' ? 'pincr' : '/efi2web_2t', done);
					break;
				case 'catalogue':
					Testable.set_product(router._path.page !== 'medium-2mt-wind30' ? 'medium-2mt-wind30' : 'medium-mslp-rain', done);
					break;
				case 's2s':
					Testable.set_product(router._path.page !== 's2s_anomaly_cawcr_2MTM' ? 's2s_anomaly_cawcr_2MTM' : 's2s_anomaly_cawcr_RAIN', done);
					break;
				default:
					Testable.test.assert(false, 'ensure_product_with_basetime: package ' + router._path.dir + ' not supported');
					break;
			}
		},

		// TODO fix changing of package
		ensure_interactive_product: function(done) {
			router.set_path('medium_range');
			done();
		},

		ensure_clickable_product: function(done) {
			var self = this, test = Testable.test;
			test._set_timeout_info('clickable product');

			User.info.then(function(user){
				test.assert(user.uid && user.uid.length > 0)
			})

			switch(router._path.dir){
				case 'catalogue':
					Testable.set_product(router._path.page !== 'medium-2mt-wind30' ? 'medium-2mt-wind30' : 'medium-mslp-rain', done);
					break;
				case 'medium_range':
					Testable.set_product(router._path.page !== 'medium-mslp-mean-spread' ? 'medium-mslp-mean-spread' : 'medium-multi-efi', done);
					break;
				case 's2s':
					Testable.test.assert(false, 'ensure_clickable_product: package ' + router._path.dir + ' does not contain clickable products');
					break;
				default:
					Testable.test.assert(false, 'ensure_clickable_product: package ' + router._path.dir + ' not supported');
					break;
			}
		},

		select_product_via_menu: function(product, done) {
			var self = this, test = Testable.test;

			Testable.ensure_product_page(function(){
				$('span:contains(Filter results)').click();
				test.wait_for('.results-menu .grid-items > div', 'menu open and populated')
					.then(function(){
						var menu = $('.results-menu');
						if(typeof product === 'number'){
							var li = menu.find('.grid-items div.item:visible:eq(' + product + ')');
							if(li.length){
								var title = li.find('.col-title').text();
								test.assert(!!title, 'failed to get title for product:' + product);
								li.click();
								test.wait_for(self.chart.element)
									.then(function(){
										test.wait_for(function(){ return $('h1:first').text() === title }, function(){ return 'Title: expected: ' + title + '. got: ' + $('h1:first').text()})
											.then(done);
									});
							}
						}
					});
			});
		},

		set_product: function(product, done) {
			console.log('%cset_product', 'background-color:#99f');
			var test = Testable.test, self = this;

			Testable.ensure_product_page(function(){
				var chart = $('.chart-column .chart');
				Testable.test.assert(chart.length !== 0, 'no charts found');
				Testable.test.assert(chart.length < 2, 'wrong number of charts:', chart.length);

				var img = chart.find('img:first').attr('src');
				switch(typeof product){
					case 'number':
						Testable.test.assert(list.filtered.length > product, 'not enough products:' + list.filtered.length);
						var p = list.items[list.filtered[product]];
						function check_title(){
							return $('h1:first').text() === p.title_();
						}

						if(check_title()){
							console.log('set_product: *** product already set:', product, ': ', p.title_());
							return done();
						}
						console.log('%cTestable.set_product: ', 'background-color:#69c', [$('h1').text()], ' --> ', [p.title_()]);

						router.set_product(p.name(), p.title_());

						// **** note that the timeout message is simultaneously set in three places so is likely to be incorrect.
						$.when(
							$.Deferred(function(df){
								Testable.test.wait_for(check_title, function(){ return 'product title: expected:', p.title_()}).then(function(){
									df.resolve();
								});
							}),
							/* the image change test is disabled for now because it is failing sometimes
							$.Deferred(function(df){
								Testable.test.wait_for(function(){ return chart.find('img:first').attr('src') !== img; }, 'image change').then(df.resolve);
							}),
							*/
							$.Deferred(function(df){
								test.wait_for(
									function(){ console.log('  *', $('.chart-column .chart'), $('.chart-column .chart').data('ui-basicChart')); return $('.chart-column .chart').length},
									'chart'
								).then(function() {
									chart = $('.chart-column .chart'); // select again, it changes!
									var bus = chart.basicChart && chart.basicChart('instance') ? chart.basicChart('instance').bus : chart.chart('instance').bus;
									bus.listen_once('chart-content-update', function(){
										var chart_image = chart.find('> img:first, .chart-image');
										test.assert(chart_image.length == 1, 'unexpected number of chart images: ' + chart_image.length);
										test.assert($('#time-navigator').length < 2, 'too many time-navigators: ' + $('#time-navigator').length);
										test.wait_for(function(){ return !$('.spinner:visible').length || $('.spinner:visible').css('opacity') < 0.0001; }, 'spinner stopped').then(function(){
											test.assert(!$('.fail').length, 'no fail icon');
											test.wait_for(function(){
												return chart_image.css('opacity') > 0.999;
											}, 'chart greyed out').then(function(){
												test.assert(chart_image.attr('name') !== 'undefined', 'img name not set');
												test.wait_for(
													function(){ console.log('       *', chart_image.attr('name'), p.name()); return chart_image.attr('name') === p.name()},
													function(){ return 'img name mismatch: got:<br/>' + chart_image.attr('name') + ', expected:<br/>' + p.name()}
												).then(df.resolve);
											});
										});
									});
								});
							})
						).then(function(){ console.log('test.set_product: done'); done(); });
						break;
					case 'string':
						test.assert(product.length > 0, 'empty product requested')
						var p = Products.product_by_name(product)
						test.assert(!!p, 'cannot find product: ' + product)
						router.set_product(product, p.title_());

						// we do not use a message here to detect when the product has been displayed
						// because when changing from one product type to another, the chart and bus will
						// be destroyed and recreated.
						test.wait_for(
							function(){
								var chart = $(self.chart.element);
								var chart_image = chart.find('> img:first, .chart-image');
								return chart_image.attr('name') === p.name()
							},
							'swap'
						).then(done);
						break;
				}
			});
		},

		set_parameter: function(parameter, value, done) {
			if(typeof value === 'number'){
				true;
			}else{
				true;
			}
			done();
		},

		change_base_time: function(value, done) {
			// value can be number, date, or null (null will increment)

			var self = this;

			function parse_date(str){
				self.test.assert(!!str, 'empty date');

				var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'July', 'Aug', 'Sept', 'Oct', 'Nov', 'Dec'];

				var d = str.split(' ');
				if(d.length == 2){
					// seasonal charts, eg 'Dec 2016'
					return new Date(d[1], months.indexOf(month), 0, 0);
				}else{
					var month = d[2].replace(',', '');
					var mm = months.indexOf(month);
					var now = new Date();
					var hour = (+d[3] === now.getFullYear()) ? d[4] : d[3];
					return new Date(now.getFullYear(), mm, d[1], hour);
				}
			}

			function selected_menu_item(menu) {
				var items = menu.find('> li');
				var index = -1;
				$.each(items, function(i, item) {
					if($(item).find('.ui-icon-check').length){
						index = i;
					}
				});
				return index;
			}

			Testable.ensure_product_with_basetime(function(){
				Testable.test.wait_for('.parameter-time', 'parameter bar')
					.then(function(){
						Testable.test.wait_for('.parameter-time > .ui-button', 'base-time menu button').then(function(){
							$('.parameter-time > .ui-button').width() // attempt to force the button to be rendered before clicking
							$('.parameter-time > .ui-button').click();
							Testable.test.wait_for('.parameter-time .parameter-menu li:visible', 'basetime menu open')
								.then(function(){
									Testable.test._set_timeout_info('changing date');
									var v = -1;

									if(value === null){
										var index = selected_menu_item($('#base_time-menu'));
										if(!!~index){
											v = index + 1;
										}
									} else if(typeof value === 'number'){
										v = value;
									}else{
										console.warn('changing menu by value not implemented');
									}

									if(!!~v){
										var item = $('#base_time-menu > li:eq(' + v + ')');
										var target_basetime = parse_date(item.text());

										// create 2 parallel promises that must both be resolved.
										Promise.all([
												new Promise(function(resolve, reject) {
													if(!$(self.chart.element).find('.basic').length){
														// interactive chart
														$(self.chart.element).messageBus().clear().listen_once('chart-content-update', function(){
															var basetime = parse_date($('.layer-title h4').text());
															if(basetime.getTime() === target_basetime.getTime()){
																resolve();
															}else{
																console.log('******* date mismatch: ', basetime, target_basetime);
															}
														});
													}else{
														// basic chart
														// TODO check correct time for basic charts
														resolve();
													}
												}),
												new Promise(function(resolve, reject) {
													item.click();
													Testable.test.wait_for_remove('.parameter-time .parameter-menu li', function(){ console.log('  basetime:', basetime.getTime(), target_basetime.getTime()); return 'menu close or date'}).then(resolve);
												})
											]).then(function(values) {
												done();
											});
									}
								});
							});
					});
			});
		},

		clear_facets: function(done){
			Testable.ensure_landing_page(function() {
				Testable.test.assert($('.filter-status').length === 1, 'cannot find filter status box');
				(function remove_box(){
					var buttons = '.filter-status > .ui-button';
					var n_buttons = $(buttons).length;
					if(n_buttons){
						$(buttons + ':first').click();
						Testable.test.wait_for(function(){ return $(buttons).length === n_buttons - 1; }, 'filter click (' + n_buttons + ')').then(remove_box);
					}else{
						done();
					}
				})();
			});
		},

		test_animation: function(done) {
			var self = this, test = Testable.test;
			test.wait_for('.button.animation-toggle', 'Play button').then(function(){
				var chart = $(Testable.chart.element);
				var src = chart.find('img:visible').attr('src');
				var h4 = chart.find('h4').text();
				test.assert(!!src, 'img');
				test.assert_no_element('.ec-toolbar .on', 'not on');
				var got_start = false;
				Bus().listen_once('animation-start', function(){
					got_start = true;
				})
				$('.button.animation-toggle').click()
				test.wait_for(function(){ return got_start }).then(function(){
					var got_stop = false;
					Bus().listen_once('animation-stop', function(){
						got_stop = true;
					})
					test.wait_for('.ec-toolbar .on', 'on').then(function(){
						test.wait_for(function(){ return chart.find('img:visible').attr('src') !== src}, 'image change').then(function(){
							if(h4) test.assert(chart.find('h4').text() !== h4, 'date title changed');
							$('.button.animation-toggle').click()
							test.wait_for(function(){ return got_stop }).then(function(){
								test.wait_for(test.not_element('.ec-toolbar .on'), 'off').then(done);
							})
						})
					})
				})
			})
		}
	}

});
