/*
 *  Widget to provide status information about current filters applied to a content list.
 */
define(['jquery', 'bus', 'virtual-dom/h'], function($, Bus, h){

var FilterStatus = function(args) {
	this.options = $.extend({
		remove: true,                     // specifies whether clicking on a filter button either removes or disables the filter.
		filters: [],
		on_disable_change: $.noop,
		on_remove: $.noop,
	}, args);
}

FilterStatus.prototype = {
	render: function() {
		var self = this, o = this.options;

		var filters = $.map(o.filters, function(f) {
			return (f.status && f.status()) ? f : null;
		});

		if(filters.length){
			return h('div.status.filter-status', $.map(filters, function(f) {
				// TODO create button widget
				return h('div.ui-button.ui-widget.ui-state-default.ui-button-text-icon-secondary' + (f.disabled ? '.disabled' : ''), {
					title: f.name + ': ' + f.status(),
					onclick: function(e) {
						if(o.remove){
							o.on_remove(f.name);
						}else{
							o.on_disable_change(f.name, f.disabled = (f.disabled ? !f.disabled : true));
						}
						e.stopPropagation()
					}
				}, [
					h('span.ui-button-text', [f.status()]),
					h('span.ui-button-icon-secondary.ui-icon.ui-icon-closethick', {title: 'clear'})
				])
			}));
		}else{
			return h('span.disabled', ['No filters applied']);
		}
	}
}

return FilterStatus;

});
