define(['jquery', 'mixins', 'filterable-list', 'virtual-dom/h', './input_filter', './scrollbar', './filter_status', './index.css'], function($, mixins, list, h, InputFilter, Scrollbar, FilterStatus){

	var VGrid = function(options){
		var self = this;

		this.options = $.extend({
			view_mode: 'groups',
			element: $(), // temporary
			height: undefined // eg '430px'
		}, options);

		this.widgets = {
			input_filter: new InputFilter({on_filter: function(text) {
				list.lookup_filter_by_name('text').enable().on_change(null, text);
			}}),
			filter_status: new FilterStatus({
				filters: list.filters,
				on_disable_change: function(filter_name, disabled){
					console.log('on_disable_change: ' + filter_name);
					var f = list.lookup_filter_by_name(filter_name);
					f.disabled = disabled;
					list.filter();
				},
				on_remove: function(filter_name){
					var filter = list.lookup_filter_by_name(filter_name);
					console.log('grid.on_remove: ' + filter_name, filter);

					filter.deactivate();
					switch(filter.name){
						case 'facets':
							list.filter();
							break;
						case 'text':
							$('.thumbnail-list-filter input').val('');
							break;
						default:
							break;
					}
				}
			}),
			scrollbar: new Scrollbar()
		}

		$.map(list.filters, this.bind(self.add_filter));

		switch(this.options.view_mode){
			case 'groups':
				this.view = require('./views/groups')
			case 'simple':
				this.view = require('./views/simple')
		}
		this.view = new this.view({
			element: this.options.element,
			on_needs_render: function() {
				list.have_filtering.then(function() {
					self.options.element.trigger('render');
				});
			},
			click: this.options.on_click,
			select: function(a){
				console.log('vgrid.select');
				list.selection.set(list.filtered[a.filtered_idx]);
			},
			scrollbar: this.widgets.scrollbar
		})
		list.have_filtering.then(function() {
			self.resize();
			self.options.element.trigger('render');
		});

		list.subscribe(self, self._on_filter = function() {
			self.view.groups_valid = false;
			self.view._positions_valid = false;
			self.options.element.trigger('render');
		})
	}

	VGrid.prototype = $.extend({
		render: function() {
			var w = this.widgets

			return h('.catalogue-grid', {style: {height: this.options.height}}, [
				h('.head', [w.input_filter.render()]),
				h('.foot', [list.filtered.length + ' matching items', w.filter_status.render()]),
				h('.outer', [
					h('.scrollable.ps-container.ps-active-y', {style: {height: '100%'}}, [
						this.view.render(),
						w.scrollbar.render()
					])
				])
			])
		},

		add_filter: function(f) {
			var self = this;

			// TODO the list should bind to the filter change events, not the grid
			if($.isFunction(f.fn)){
				f.events && f.events.bind('changed', function(){
					list.filter();
				});
			}
		},

		resize: function() {
			this.view.resize();
		},
	}, mixins)

	return VGrid;
})
