/*
 *  A wintersmith Content plugin for prerendering a product browser page.
 *
 *  - inherits from the Page class.
 *  - is triggered for pages of type .browser.yaml
 *
 */
(function() {
	var extend = function(child, parent) {
		for (var key in parent) {
			if (hasProp.call(parent, key)) child[key] = parent[key];
		}
		function ctor() { this.constructor = child; }
		ctor.prototype = parent.prototype;
		child.prototype = new ctor();
		child.__super__ = parent.prototype;
		return child;
	};
	var hasProp = {}.hasOwnProperty;

	var async = require('async');
	var fs = require('fs');
	var path = require('path');
	var url = require('url');
	var yaml = require('js-yaml');
	var request = require('sync-request');
	var jsdom = require('jsdom').jsdom;
	var beautify = require('js-beautify').html;

	module.exports = function(env, callback) {
		var view = function(env, locals, contents, templates, callback) {
			var self = this;
			var exclude = [
				/*
				'forecasts/charts/all/index.html',
				'forecasts/charts/catalogue/index.html',
				'forecasts/charts/obstat/index.html',
				'forecasts/charts/mofc_multi/index.html',
				'forecasts/charts/long-range/index.html',
				'forecasts/charts/s2s/index.html',
				'forecasts/charts/catalogue/w_t2m30mw.html',
				'forecasts/charts/catalogue/w_z500t850.html'
				*/
			]
			// TODO check timestamps
			console.log('>>>>>>>>>>>>> prerendering: ', this.filename, exclude.indexOf(this.filename) > -1 ? '[exluding]' : '');

			var template = templates[path.normalize(this.template)];
			ctx = {page: this};
			env.utils.extend(ctx, locals);
			var html = template.render(ctx, function(err, html){
				if(err) return console.warn('chart view:', err), callback(err, null);
				if(exclude.indexOf(this.filename) > -1) return callback(null, html);
				try {
					self.pre_render(html.toString(), function(e, html){
																			console.log('>>>>>>>>>>>>> prerender view: prerender done');
						callback(null, new Buffer(html, 'utf8'));
					});
				} catch(e){
						callback(e, html);
				}
			});

			return html;
		}

		var ChartPage = (function(superClass) {
			extend(ChartPage, superClass);

			function ChartPage(filepath1, metadata1, markdown1) {
				console.log('      ChartPage: filepath=', filepath1.relative);
				var y = yaml.safeLoad(fs.readFileSync(filepath1.full, 'utf8'));;


				//var res = request('GET', y.url + '?_pjax=true', {headers: { 'X-PJAX': '1' }});
				/*
				console.log('-----------------------------------------------------------');
				console.log('-----------------------------------------------------------');
				console.log(res.getBody().toString());
				console.log('-----------------------------------------------------------');
				console.log('-----------------------------------------------------------');
				*/

				this.filepath = filepath1;
				this.metadata = {template: 'page.jade'};
				extend(this.metadata, metadata1);
				extend(this.metadata, y);
				this.markdown = 'Chart page';
			}

			/*
			 * At present jsdom does not handle navigation (such as setting window.location.href === "https://example.com/"). However, if you'd like to change the URL of an existing Window instance (such as for testing purposes), you can use the jsdom.changeURL method:
          *   jsdom.changeURL(window, "https://example.com/");
			 */

			ChartPage.prototype.pre_render = function(html, callback) {
				console.log('* starting jsdom...');
				html = html.replace(/<script.*\/script>/g, ''); // TODO why do scripts stop jsdom page loading?

				var window = jsdom(html, {
					features: {
						FetchExternalResources: ['script'], // this is the default
						ProcessExternalResources: ['script'],
						MutationEvents: '2.0'
					},
					//documentRoot: '/var/tmp/sdo/git/webdev/apps/frontend/static/frontend',
// TODO - host
					//url: 'http://atrebates.ecmwf.int:8000/forecasts/charts/medium/',
					url: 'http://apps-test.ecmwf.int:8000/' + this.filename.replace('index.html', ''),
					virtualConsole: require('jsdom').createVirtualConsole().sendTo(console),
					created: function(errors, window) {
						console.log('jsdom.created');
					},
					onload: function() {
						console.log('jsdom.onload');
					},
					done: function(errors, window) {
						console.log('jsdom.done');
					}
				}).defaultView;

				function on_error(e) {
					console.error('Error occured: ', e.error.message);
					console.error('Error occured: ', e.error.stack);
				}

				window.addEventListener('error', on_error)

				window.addEventListener('load',  function(){
					window.is_node = true;
					var document = window.document; // document is not available initially.
					/*
					window.jQuery('script').load(function() {
						console.log('--> script loaded!', this.src);
					});
					*/

					// the js has already been created, so as long as the server is already running we should be able to load the js
					var scriptEl = document.createElement('script');
					scriptEl.src = 'file://' + __dirname + '/../../src/contents/js/iso/product_browser.js';
					//scriptEl.src = 'http://solrdev.ecmwf.int:89/js/static_charts.js';
					document.body.appendChild(scriptEl);

					function finished() {
						console.log('-----------------------------------');

						if(false){
							var scripts = document.getElementsByTagName('script');
							if(scripts.length){
								console.log('scripts:');
								for(var i=0;i<scripts.length;i++) {
									if(scripts[i].src) console.log(i, scripts[i].src)
									else console.log(i, scripts[i].innerHTML)
								}
							}else{
								console.warn('no scripts');
							}
						}

						// these are all inline styles - it should be safe to delete them
						Array.prototype.slice.call(document.getElementsByTagName('style')).forEach((a, b) => {
							a.remove();
						});

						// remove in-progress jsonp calls and file:/// scripts
						Array.prototype.slice.call(document.getElementsByTagName('script')).forEach((a) => {
							/*
							Array.prototype.slice.call(a.attributes).forEach((a) => {
								console.log('      - attr=' + a.nodeName + '=' + a.nodeValue);
							});
							console.log('     src=', a.attributes.src);
							*/
							if(a.attributes.src){
							if(!!~a.attributes.src.nodeValue.indexOf('callback=')){
								a.remove();
							}
							if(!!~a.attributes.src.nodeValue.indexOf('file://')){
								a.remove();
							}
							}
						});

						// store product list and thumbnails
						window.get_cachable_data(function(data){
							console.log('data=', data)
							var script = document.createElement('script');
							script.innerHTML = '_ECMWF = window._ECMWF || {};\n_ECMWF.products = ' + data + ';'
							document.head.appendChild(script);
						});

						function remove(selector) {
							var el = document.querySelectorAll(selector)[0];
							el && el.remove();
						}
						function remove_attr(selector, attr) {
							var el = document.querySelectorAll(selector)[0];
							el && el.removeAttribute(attr);
						}
						remove('.col2-control');
						remove('.catalogue-grid .ps-scrollbar-x-rail');
						remove('.catalogue-grid .ps-scrollbar-y-rail');
						remove('.thumb-progress-bar');
						remove('article');

						// prevent the height of the grid being set to zero
						remove_attr('.catalogue-grid .scrollable', 'style');
						remove_attr('.catalogue-grid .scrollable', 'sizor');
						remove_attr('.catalogue-grid .scrollable', 'max-sizor');

						// we want to disable log messages now, but dont seem to be able
						/*
						var virtualConsole = require('jsdom').getVirtualConsole(window);
						console.log('virtualConsole=', virtualConsole);
						virtualConsole._events.error = function(){};
						virtualConsole._events.log = function(){};
						console.log('virtualConsole=', virtualConsole._events.Console);
						virtualConsole._events.Console = function dummy(){};
						console.log('virtualConsole=', virtualConsole._events.Console);
						*/

						var html = document.documentElement.outerHTML;
						window.removeEventListener('error', on_error);
						// put back scripts that were removed ?
						html = '<!DOCTYPE html>\n' + html.replace(/<\/head>/, '<script src="/js/builds/common.js"></script>\n  <script src="/js/builds/product_browser.js"></script>\n  </head>');
						callback(null, beautify(html, {indent_size: 2}));

						window.close();
					}
// TODO find better way of detecting finish.
// TODO cancel outstanding ajax requests to prevent errors being displayed?
					setTimeout(function() {
						finished();
					}, 10000);
				});
				/*
				window.addEventListener('DOMContentLoaded', function() {
					console.log('--> dom loaded!');
				});
				*/
			}

			/*
			 *  use a custom view that calls jsdom after the template is loaded
			 */
			ChartPage.prototype.getView = function() {
				return this.metadata.view || 'prerender';
			}

			ChartPage.prototype.getLocation = function(base) {
																					console.log('***** getLocation');
				var uri = this.getUrl(base);
				return uri.slice(0, +uri.lastIndexOf('/') + 1 || 9e9);
			}

			ChartPage.prototype.getHtml = function(base) {
				if (base == null) {
					base = env.config.baseUrl;
				}

				return this.markdown;
			}

			/*
			ChartPage.prototype.getUrl = function(base) {
				return 'forecasts/charts/medium/index.html';
			}
			*/
			ChartPage.prototype.getFilename = function() {
				filename = ChartPage.__super__.getFilename.call(this).replace(/.browser.html$/, '.html');
				//																	console.log('***** getFilename', filename);
				return filename;
			}

			return ChartPage;

		})(env.plugins.Page);

		ChartPage.fromFile = function(filepath, callback) {
			return async.waterfall([
				function(callback) {
					return fs.readFile(filepath.full, callback);
				},
				function(buffer, callback) {
					return ChartPage.extractMetadata(buffer.toString(), callback);
				},
				(function(_this) {
					return function(result, callback) {
						var markdown = result.markdown;
						var metadata = result.metadata;
						var page = new _this(filepath, metadata, markdown);
						return callback(null, page);
					};
				})(this)
			], callback);
		};

		ChartPage.extractMetadata = function(content, callback) {
			var end, markdown, metadata = '', parseMetadata, result;

			function parseMetadata (source, callback) {
				var error, error1, lines, markerPad;
				if (!(source.length > 0)) {
					return callback(null, {});
				}
				try {
					return callback(null, yaml.load(source) || {});
				} catch (error1) {
					error = error1;
					if ((error.problem != null) && (error.problemMark != null)) {
						lines = error.problemMark.buffer.split('\n');
						markerPad = ((function() {
							var i, ref, results;
							results = [];
							for (i = 0, ref = error.problemMark.column; 0 <= ref ? i < ref : i > ref; 0 <= ref ? i++ : i--) {
								results.push(' ');
							}
							return results;
						})()).join('');
						error.message = "YAML: " + error.problem + "\n\n" + lines[error.problemMark.line] + "\n" + markerPad + "^\n";
					} else {
						error.message = "YAML Parsing error " + error.message;
					}
					return callback(error);
				}
			};

			markdown = content;
			if (content.slice(0, 3) === '---') {
				result = content.match(/^-{3,}\s([\s\S]*?)-{3,}(\s[\s\S]*|\s?)$/);
				if ((result != null ? result.length : void 0) === 3) {
					metadata = result[1];
					markdown = result[2];
				}
			} else if (content.slice(0, 12) === '```metadata\n') {
				end = content.indexOf('\n```\n');
				if (end !== -1) {
					metadata = content.substring(12, end);
					markdown = content.substring(end + 5);
				}
			}
			return async.parallel({
				metadata: function(callback) {
					return parseMetadata(metadata, callback);
				},
				markdown: function(callback) {
					return callback(null, markdown);
				}
			}, callback);
		};

		env.registerView('prerender', view);
		// ******************* temporary - keep them separate for debugging
		env.registerContentPlugin('browser', '**/*.browser.yaml', ChartPage);
		return callback();
	};

}).call(this);
